/**
 * CBOS - Coffe Bean Ordering System
 * ICS52 - Software Engineering - University of California - Irvine
 * Fall 2000
 * 
 * @author Arvind Krishna
 * @author Roberto Silveira Silva Filho 
 * @version 1.0
 *
 */

package cbos;

import cbos.client.*;
import cbos.server.*;

import javax.swing.JTabbedPane;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;

import java.awt.*;
import java.awt.event.*;

import java.io.*;       // Manages file save process
import java.util.zip.*; // Read/write of compressed files


public class CBOS extends JPanel implements java.lang.Runnable {
	
	ServerFactory serverFactory = null;
	static CBOS cbosInstance = null;
	private boolean print = false;
	
	
	private PurchaseClientInterface purchaseClient = null;
	private SaleClientInterface saleClient = null;
	private WarehouseClientInterface purchaseWarehouseClient = null;
	private WarehouseClientInterface saleWarehouseClient = null;
	private ReportsInterface reportsClient = null;
	
	private String uid; // User id provided by the user. May not be the right one.
	
	
	public CBOS() {	
		String userId = loginUser();
		uid = userId;
		accessTheSystem(userId);
		
	}
	
	public void run() {
		//accessTheSystem(uid);
	}
	
	/**
	 * Access the main system window after login
	 */
	public void accessTheSystem(String login) {
		
		constructComponents(login);
		formatGUI();
		runScheduler();
		
	}
	
	/**
	 * Displays the window of login
	 */
	private String loginUser() {
		String id;
		boolean validated = false;
		cbos.server.Log log;
		
		id = "";
		
		while (id.length() != 8 ) {
			
			id = JOptionPane.showInputDialog(this,"Input your Student ID");
			
			
			if (id.length() != 8 ) {
				JOptionPane.showMessageDialog(this,"Your user ID must have 8 characters!!","Wrong Size", JOptionPane.ERROR_MESSAGE);
				id = "";
			} 
			
		}
		
		return id;
	}
	
	/**
	 * Format the GUI components to be shown
	 */
	public void formatGUI() {
		if (print)
			System.out.println("CBOS: Formating GUI...");
		
		JTabbedPane tabbedPane = new JTabbedPane();
		
		Component introPanel = makeIntroductionPanel();
		tabbedPane.addTab("Introduction", null, introPanel, "Introduces the system");
		tabbedPane.setSelectedIndex(0); // Selects the first panel
		
		Component purchaseWarehousePanel = makePurchaseWarehousePanel();
		tabbedPane.addTab("PurchaseWarehouse", null, purchaseWarehousePanel, "Handles the purchase warehouse information");
		
		Component saleWarehousePanel = makeSaleWarehousePanel();
		tabbedPane.addTab("SaleWarehouse", null, saleWarehousePanel, "Handles the sales warehouse information");
		
		Component buyPanel = makePurchasePanel();
		tabbedPane.addTab("Purchase", null, buyPanel, "Handles the purchase orders");
		
		Component sellPanel = makeSalePanel();
		tabbedPane.addTab("Sell", null, sellPanel, "Handles the sales orders");

		Component reportsPanel = makeReportsPanel();
		tabbedPane.addTab("Reports", null, reportsPanel, "Handles the report generation");
				
		Component dataEntryPanel = makeDataEntryPanel();
		tabbedPane.addTab("DataEntry", null, dataEntryPanel, "Handles the the input of auxiliary data");
		
		if (print)
			System.out.println("Finished formating all windows, now showing...");
		
		//Add the tabbed pane to this panel.
		setLayout(new GridLayout(1, 1)); 
		add(tabbedPane);
	}

	
	private void constructComponents(String login) {
		
		serverFactory = new ServerFactory();
		serverFactory.setLogin(login);
		
		// 14 errors will be inserted.
		// using provided student id..
		serverFactory.createServerSide(login, 14);
		
	}
	
	
	private void runScheduler() {
		
	}
	
	/**
	 * Save a provided object to a file.
	 * @param object the object reference to be serialized
	 * @param filename the name of the file to receive the serialized stream data
	 * @param dirname the name of the directory to hold the file
	 */
	private void saveObject(Object object, String filename, String dirname) {
			
		if (filename != null) {
			
			try {

				File directory = new File(dirname);
				if (! directory.exists()) {
					directory.mkdir();
				}
				
				File file = new File(directory, filename);
				FileOutputStream fos = new FileOutputStream(file);
				GZIPOutputStream gzos = new GZIPOutputStream(fos);
				ObjectOutputStream out = new ObjectOutputStream(gzos);
				
				out.writeObject(object);
				
				out.flush();
				out.close();
				
			} catch ( IOException e ) {
				System.out.println("LOA: Error while serializing to file: "+dirname+filename);
				System.out.println(e);
			}
		}
	}
	
	/**
	 * Recursively saves the system data in their respective files.
	 */
	private void saveSystem() {
	
		if (serverFactory != null) {
			serverFactory.saveAll();
		}
	}
	
	
	protected Component makeIntroductionPanel() {
		if (print)
			System.out.println("CBOS: creating IntroductionPanel...");
		
		IntroductionClient panel = new IntroductionClient(serverFactory);
		return (JPanel) panel;
	}
	
	/**
	 * Buy form
	 */
	protected Component makePurchasePanel() {
		if (print)
			System.out.println("CBOS: creating PurchasePanel...");
		
		PurchaseClient panel = new PurchaseClient(serverFactory, purchaseWarehouseClient);
		purchaseClient = panel;
		return (JPanel) panel;
	}
	
	protected Component makeSalePanel() {
		if (print)
			System.out.println("CBOS: creating SalePanel...");
		
		SaleClient panel = new SaleClient(serverFactory, saleWarehouseClient);
		saleClient = panel;
		return (JPanel) panel;
	}
	
	protected Component makeReportsPanel() {
		if (print)
			System.out.println("CBOS: creating ReportsClient...");
		
		ReportsClient panel = new ReportsClient(serverFactory);
		reportsClient = panel;
		return (JPanel) panel;
	}
	
	protected Component makePurchaseWarehousePanel() {
		if (print)
			System.out.println("CBOS: creating WarehouseClient...");
		
		PurchaseWarehouseClient panel = new PurchaseWarehouseClient(null,null, serverFactory);
		purchaseWarehouseClient = panel;
		return (JPanel) panel;
	}
	
	protected Component makeSaleWarehousePanel() {
		if (print)
			System.out.println("CBOS: creating SalesWarehousePanel...");
		
		SalesWarehouseClient panel = new SalesWarehouseClient(null,null, serverFactory);
		saleWarehouseClient = panel;
		return (JPanel) panel;
	}
	
	protected Component makeDataEntryPanel() {
		if (print)
			System.out.println("CBOS: creating DataEntryPanel...");
		
		DataEntryClient panel = new DataEntryClient(serverFactory, purchaseClient,
													saleClient, purchaseWarehouseClient,
													saleWarehouseClient, reportsClient);
		return (JPanel) panel;
	}
	
	public static void main(String[] args) {
		
		FinishThread killThread;
		boolean print = false;
		
		JFrame frame = new JFrame("CBOS");
		//JScrollPane scrollPane = new JScrollPane(frame);
		cbosInstance = new CBOS();
		
		frame.getContentPane().add(cbosInstance,BorderLayout.CENTER);
		frame.setSize(640,580);
		frame.setVisible(true);
		//frame.setName("CBOS "+cbosInstance.uid);
		frame.setTitle("CBOS - UID: "+cbosInstance.uid);
		
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				cbosInstance.saveSystem();
				/*
				try {
					System.in.read();
				} catch (IOException ex) {
					System.out.println(ex);	
				}
				*/
				
				System.exit(0);}
			});
		
		if (print)
			System.out.println("CBOS: running the FinishThread...");
		killThread = new FinishThread(cbosInstance);
	

	}
	
	public void finish() {
		Log log = serverFactory.getLog();
		log.submitBug(uid,13);
		System.out.println("CBOS: PROGRAM CRASHED DUE TO A BUG :-) !!!"); 
		System.exit(0);
	}
}