/**
 * CBOS - Coffe Bean Ordering System
 * ICS52 - Software Engineering - University of California - Irvine
 * Fall 2000
 * 
 * @author Arvind Krishna
 * @author Roberto Silveira Silva Filho 
 * @version 1.0
 *
 */
package cbos.client;

import java.util.Vector;
import javax.swing.table.AbstractTableModel;

public class CBOSTableModel extends AbstractTableModel  {
	
	private Vector columnNames = new Vector(0,1);
	private Vector data        = new Vector(0,1);
	private Vector columnTypes = new Vector(0,1); // Class vector
	private boolean print      = false;
	
	private int numColumns = 0; // Number of columns in the table
	
	/**
	 * Constructor
	 * @param colNames Names of the columsn of the table
	 * @param tableData the data to be presetned in the table
	 */
	public CBOSTableModel(Vector colNames, Vector tableData) {
		
		if (print) {
			System.out.println("CBOSTableModel: Initializing TableModel...");
			System.out.println(colNames);			
		}
		
		if (columnNames != null)
			columnNames = colNames;
		
		if (tableData != null)
			data = tableData;
		
		numColumns = colNames.size();
		fireTableChanged(null);
	}
	
	/**
	 * Constructor
	 * @param colNames Names of the columsn of the table
	 * @param tableData the data to be presetned in the table
	 */
	public CBOSTableModel(Vector colNames, Vector tableData, Vector types) {
		this (colNames, tableData);
		columnTypes = types;
		
	}
	
	
	/**
	 * Adds a new line to the table
	 */
	public void addNewLine() {
		
		if (print){
			System.out.println("CBOSTableModel: Inserting new Blank Line...");
		}
		
		for (int t=0; t<numColumns; t++) {
			data.addElement(new String(""));
		}
		
		if (numColumns != 0) {
			int newRow = data.size()/numColumns - 1;
			fireTableRowsInserted(newRow, newRow);
		}
		
	}
	
	/**
	 * Removes a line from the table
	 */
	public void removeLine(int line) {
		int columns = columnNames.size();
		
		if (print){
			System.out.println("CBOSTableModel: Removing Line "+line);
		}
		
		if (data.size() >0 && line>=0 && line < data.size()){
			int removedRow = 0;
			
			if (numColumns != 0)
				removedRow = data.size()/numColumns - 1;
			
			for (int t=0; t<columns; t++) {
				data.removeElementAt(line*columns);
			}
			fireTableRowsDeleted(removedRow, removedRow);
		}
	}
	
	/**
	 * Return the number of rows in the table
	 */
	public int getRowCount() {
		int columnSize = columnNames.size();
		
		if (columnSize >0)
			return data.size()/columnSize;
		else 
			return 0;
	}

	/**
	 * Returns the number of columns in the table
	 */
	public int getColumnCount() {
		return columnNames.size();
	}
	
	/**
	 * Returns the label (name) of the provided column
	 */
	public String getColumnName(int col) {
		
		if (print)
		{	System.out.println("CBOSTableModel: Getting column name in col "+col);
		}
		
		return (String) columnNames.elementAt(col);
	}

	/**
	 * Return the value at the provided position
	 * @param row - The line where the element is
	 * @param col - The column of th element
	 */
	public Object getValueAt(int row, int col) {
		
		if (print){
			System.out.println("CBOSTableModel: Getting value at "+row+","+col);
		}
		if (data.size() > row*numColumns + col-1)
			return data.elementAt(row*numColumns + col);
		else
			return null;
	}

	/**
	 * Return the type (class) of the objects in one column
	 */
	public Class getColumnClass(int c) {
		
		if (print){
			System.out.println("getColumnClass called...");
		}
		
		if ((columnTypes == null) || (c > columnTypes.size()) ) {
			return Object.class;
		} else if (columnTypes.size() >0) {
			return (Class) columnTypes.elementAt(c);
		} else {
			return Object.class;
		}
		
	}
		
	/*
	* Don't need to implement this method unless your table's
	* editable.
	*/
	
	public boolean isCellEditable(int row, int col) { 
		//Note that the data/cell address is constant,
		//no matter where the cell appears onscreen.            
		return true;
	}
	/*
	* Don't need to implement this method unless your table's
	* data can change.
	*/
	public void setValueAt(Object value, int row, int col)
	{
		int position = row*numColumns+col;
		
		if (position <data.size()) {
			data.removeElementAt(position);
			data.insertElementAt(value, position);
		
			fireTableCellUpdated(row, col);
		}
		
	} 
	/**
	 * Clear the table
	 */
	public void setNull()
	{
		int numberOfRows = getRowCount();
		
		//columnNames = new Vector(0,1);
		data = new Vector(0,1);
		fireTableRowsDeleted(0, numberOfRows);
		//fireTableChanged(null);
		
	}
	
} // end of the class defenition 