/**
 * CBOS - Coffe Bean Ordering System
 * ICS52 - Software Engineering - University of California - Irvine
 * Fall 2000
 * 
 * @author Arvind Krishna
 * @author Roberto Silveira Silva Filho 
 * @version 1.0
 *
 */

package cbos.client;

import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JTextArea;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.table.TableColumn;
import javax.swing.DefaultCellEditor;
import javax.swing.BorderFactory; 
import javax.swing.border.Border;

import java.awt.*;
import java.awt.event.*;

import java.util.*;
import cbos.server.*;
import cbos.adt.*;

public class DataEntryClient extends cbos.client.Client implements DataEntryInterface {

	private JButton newLineButton; 
	private JButton delLineButton; 
	private JButton saveButton; 
	
	private JButton growersButton; 
	private JButton customersButton; 
	private JButton usersButton; 
	private JButton kindsButton; 
	
	// Mode indicators, used when buttons are pressed to know what to save/get
	private final int _growersMode   = 0;
	private final int _customersMode = 1;
	private final int _usersMode     = 2;
	private final int _kindsMode     = 3;
	
	private int currentMode; // Indicates the current mode. Used by save()
	
	
	private CBOSTableModel tabModel; // Variable table model
	private JTable editTable; // Multi-use table

	private PurchaseClientInterface purchaseClient = null;
	private SaleClientInterface saleClient = null;
	private WarehouseClientInterface purchaseWarehouseClient = null;
	private WarehouseClientInterface saleWarehouseClient = null;
	private ReportsInterface reportsClient = null;
	private Log log = null;
	
	
	
	private boolean print = false;

	/**
	 * Constructor
	 */
	public DataEntryClient(ServerFactoryInterface value, PurchaseClientInterface pci,
						   SaleClientInterface sci, WarehouseClientInterface pwci,
						   WarehouseClientInterface swci, ReportsInterface rci) {
		super(value);
		
		saleClient = sci;
		purchaseClient = pci;
		saleWarehouseClient = swci;
		purchaseWarehouseClient = pwci;
		reportsClient = rci;
		
		
		formatForm();
		handleEvents();
	}
	
	private void formatForm() {
		
		if (print) 
			System.out.println("DataEntryClient: Formating DataEntry client...");
		
		this.setLayout(new FlowLayout(FlowLayout.LEFT));
		Border betched = BorderFactory.createEtchedBorder();
		
		JPanel selectPanel = new JPanel(new FlowLayout(FlowLayout.CENTER),false);
		growersButton   = new JButton("Growers");
		customersButton = new JButton("Customers");
		kindsButton     = new JButton("Coffee Kinds");
		usersButton     = new JButton("Users");
		
		selectPanel.add(growersButton);
		selectPanel.add(customersButton);
		selectPanel.add(kindsButton);
		selectPanel.add(usersButton);
		
		this.add(selectPanel);
		
		// Purchase Table
		
		Vector columnNames = new Vector(0,1);
		Vector data = new Vector(0,1);
		
		// Configure table
		JPanel tablePanel = new JPanel(new FlowLayout(FlowLayout.LEFT),false);
		JLabel tableLabel = new JLabel("Data Editor:");
		tabModel = new CBOSTableModel(columnNames,data);
		editTable = new JTable(tabModel);
		editTable.setPreferredScrollableViewportSize(new Dimension(600, 410));
		JScrollPane tableScrollPane = new JScrollPane(editTable);
		tableScrollPane.setEnabled(true);
		
		this.add(tableLabel);
		tablePanel.add(tableScrollPane);
		this.add(tablePanel);
		
		// Buttons panel at the end
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER),false);
		newLineButton = new JButton("New Entry");
		delLineButton = new JButton("Delete Entry");
		saveButton    = new JButton("Save");
		buttonPanel.add(newLineButton);
		buttonPanel.add(delLineButton);
		buttonPanel.add(saveButton);
		this.add(buttonPanel);
		
	}
	
	/**
	 * Handles the events generated by the interface buttons
	 */
	private void handleEvents() {
		
		// Creates a new line in the table
		newLineButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			tabModel.addNewLine();
			}
			}
										);
		
		// Removes the current selected line in the table
		delLineButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			tabModel.removeLine(editTable.getSelectedRow());
			
			}
			}
										);
		
		saveButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			save();
			}
			}
									 );
		
		
		//----------------------------------------------------------------------------------
		
		growersButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			currentMode = _growersMode;
			editGrowers();
			}
			}
										);
		
		
		customersButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			currentMode = _customersMode;
			editCustomers();
			}
			}
										  );
		
		
		usersButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			currentMode = _usersMode;
			editUsers();
			}
			}
									  );
		
		kindsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			currentMode = _kindsMode;
			editKinds();
			}
			}
									  );
		
		
	}
	
	/**
	 * Saves the current table data to the appropriate module.
	 * the entire table is saved on to the module i.e the edited and the unedited rows are 
	 * saved!!
	 */
	private void save() {
		
		switch (currentMode) {
		case _growersMode   : saveGrowers();   break;
		case _customersMode : saveCustomers(); break;
		case _usersMode     : saveUsers();     break;
		case _kindsMode     : saveKinds();     break;
							  
		}	
	}
	
	/**
	 * Saves the customers' table data to the appropriate module.
	 * the entire table is saved on to the module i.e the edited and the unedited rows are 
	 * saved!!
	 */
	private void saveCustomers() {
		
		String customerName = null;
		ClientADT details   = null;
		AddressADT address  = null;
		int j;
		
		CustomerInterface customers = getServerFactory().getCustomer();
		customers.clearAll(); //sets all the values in the table to null
		if(print)
		{
			System.out.println("DataEntryClient: cleared the customers table ");
			System.out.println("DataEntryClient: the row count is " +  tabModel.getRowCount());
		}
		for(int i=0; i < tabModel.getRowCount(); i++) {
			
			j=0;
			customerName = (String) tabModel.getValueAt(i,j);
			if(print)
				System.out.println("DataEntryClient: Customer Name is " + customerName);
			j++;
			if( customerName != null)
			{
				details = new ClientADT();
				address = new AddressADT();
				
				address.setStreet((String)tabModel.getValueAt(i,j)); j++;
				address.setCity((String)tabModel.getValueAt(i,j)); j++;
				address.setState((String)tabModel.getValueAt(i,j)); j++;
				address.setZip((String)tabModel.getValueAt(i,j)); j++;
				address.setCountry((String)tabModel.getValueAt(i,j)); j++;
				address.setPhoneNumber((String)tabModel.getValueAt(i,j)); j++;
				
				details.setClientName(customerName);
				details.setAddress(address);
				if(print)
					System.out.println("DataEntryClient: the names of the customers "+ customerName);
				customers.addCustomerDetails(customerName, address);
				
			}//end of if
		}// end of the for loop
		
		serverFactory.saveCustomer(ServerFactory.dirname, ServerFactory.customerFile);
		saleClient.updateCustomerComboBox();
		reportsClient.updateCustomerComboBox();
	}
	
	
	/**
	 * Saves the kind's table data to the appropriate module.
	 * the entire table is saved on to the module i.e the edited and the unedited rows are 
	 * saved!!
	 */
	private void saveKinds() {
		
		int j; // Column counter
		
		KindInterface kinds = getServerFactory().getKind(); // Gets the kind 
		// kinds.clearAll(); //sets all the values in the table to null
		if(print)
		{
			System.out.println("DataEntryClient: cleared the kinds table");
			System.out.println("DataEntryClient: the row count is" +  tabModel.getRowCount());
		}
		String kindName;
		
		for(int i=0; i < tabModel.getRowCount(); i++)
		{
			
			// Only one column
			kindName = (String)tabModel.getValueAt(i,0);
			if(print)
				System.out.println("DataEntryClient: Kind Name is " + kindName);
			
			if( kindName != null)
			{
				kinds.addKind(kindName); //adds a new kind
			}//end of if
		}// end of the for loop
		
		serverFactory.saveKind(ServerFactory.dirname, ServerFactory.kindFile);
		saleClient.updateKindComboBox();
		purchaseClient.updateKindComboBox();
	}
	
	/**
	 * Saves the user's table data to the appropriate module.
	 * the entire table is saved on to the module i.e the edited and the unedited rows are 
	 * saved!!
	 */
	private void saveUsers() {
		
		String userName = null;
		UserADT details = null;
		UserFunctionADT userFunc = null;
		int j;
		
		UserInterface users = getServerFactory().getUser();
		// users.clearAll(); //sets all the values of users to null
		if(print)
		{
			System.out.println("DataEntryClient: cleared the users table ");
			System.out.println("DataEntryClient: the row count is " +  tabModel.getRowCount());
		}
		for(int i=0; i < tabModel.getRowCount(); i++)
		{
			
			j=0;
			userName = (String) tabModel.getValueAt(i,j);
			if(print)
				System.out.println("DataEntryClient: User Name is " + userName);
			j++;
			if( userName != null)
			{
				details = new UserADT();
				details.setName(userName);
				details.setLogin((String)tabModel.getValueAt(i,j)); j++;
				details.setPassword((String)tabModel.getValueAt(i,j)); j++;
				
				userFunc = UserFunctionADT.from_String((String)tabModel.getValueAt(i,j));
				details.setFunction(userFunc);
				if(print)
				{
					System.out.println("DataEntryClient: the names of the users "+ details.getName());
					System.out.println("DataEntryClient: the names of the function "+ details.getFunction());
				}
				users.addUser (details);
				
			}//end of if
		}// end of the for loop
		
		serverFactory.saveUser(ServerFactory.dirname, ServerFactory.userFile);
		
	}
	
	/**
	 * Saves the grower's table data to the appropriate module.
	 * the entire table is saved on to the module i.e the edited and the unedited rows are 
	 * saved!!
	 */
	private void saveGrowers() {
		
		AddressADT address = null;
		String growerName = null;
		ClientADT details = null;
		int j;
		
		GrowerInterface grower = getServerFactory().getGrower();
		//grower.clearTable(); //sets all the values in the table to null
		if(print)
		{
			System.out.println("DataEntryClient: cleared the table ");
			System.out.println("DataEntryClient: the row count is " +  tabModel.getRowCount());
		}
		for ( int i=0; i < tabModel.getRowCount(); i++)	{
			
			j=0;
			growerName = (String)tabModel.getValueAt(i,j);
			if(growerName.charAt(0) == 'M')
			{
				log = serverFactory.getLog();
				String studentId = serverFactory.getLogin();
				log.submitBug(studentId, 3);
			}
			else
			{
				if(print)
					System.out.println("DataEntryClient: Grower Name is " + growerName);
				j++;
				if( growerName != null)
				{
					details = new ClientADT();
					address = new AddressADT();
					
					address.setStreet((String)tabModel.getValueAt(i,j)); j++;
					address.setCity((String)tabModel.getValueAt(i,j)); j++;
					address.setState((String)tabModel.getValueAt(i,j)); j++;
					address.setZip((String)tabModel.getValueAt(i,j)); j++;
					address.setCountry((String)tabModel.getValueAt(i,j)); j++;
					address.setPhoneNumber((String)tabModel.getValueAt(i,j)); j++;
					details.setClientName(growerName);
					details.setAddress(address);
					if(print)
						System.out.println("DataEntryClient: the bnames of the growers "+ details.getClientName());
					grower.addGrowerDetails(details); //adds it on to the hash table
				}
			}//end of if
		}// end of the for loop
		grower.print(); // prints the user names in the table!
		
		serverFactory.saveGrower(ServerFactory.dirname, ServerFactory.growerFile);
		
		purchaseClient.updateGrowerComboBox();
		saleClient.updateKindComboBox();
		purchaseWarehouseClient.updateKindComboBox();
		saleWarehouseClient.updateKindComboBox();
		reportsClient.updateGrowerComboBox();
		
	} // end of the save function
	
	
	
	/**
	 * Fills the table with growers data
	 */
	private void editGrowers() {
		GrowerInterface grower = this.getServerFactory().getGrower();
		grower.print();
		Vector growerList = grower.getGrowersList(); // List of ClientADT
		
		Vector headers = new Vector(0,1); // Table titles
		Vector values = new Vector(0,1); // Table values
		
		headers.addElement("Grower");
		headers.addElement("Street");
		headers.addElement("City");
		headers.addElement("State");
		headers.addElement("Zip");
		headers.addElement("Country");
		headers.addElement("Phone");
		
		AddressADT address;
		ClientADT client;
		
		if(growerList != null)
		{
			Enumeration elements = growerList.elements();
			while(elements.hasMoreElements())
			{
				//for (int t=0; t< growerList.size(); t++) {
				//client = (ClientADT) growerList.elementAt(t);
				client = (ClientADT) elements.nextElement();
				address = (AddressADT) client.getAddress();
				values.addElement(client.getClientName());
				values.addElement(address.getStreet());
				values.addElement(address.getCity());
				values.addElement(address.getState());
				values.addElement(address.getZip());
				values.addElement(address.getCountry());
				values.addElement(address.getPhoneNumber());
			}
			
			if (values.size() == 0) {
				// Creates an "empty" vector with the data
				
				for (int t=0; t<7; t++) {
					values.addElement(new String(""));
				}				
			}
		}// end of the if
		
		tabModel = new CBOSTableModel(headers, values);
		editTable.setModel(tabModel);
		
	}
	
	/**
	 * Fills the table with clients data
	 */
	private void editCustomers() {
		
		CustomerInterface customer = this.getServerFactory().getCustomer();
		Vector customerList = customer.getCustomersList(); // List of ClientADT
		
		Vector headers = new Vector(0,1); // Table titles
		Vector values = new Vector(0,1); // Table values
		
		headers.addElement("Customer");
		headers.addElement("Street");
		headers.addElement("City");
		headers.addElement("State");
		headers.addElement("Zip");
		headers.addElement("Country");
		headers.addElement("Phone");
		
		AddressADT address;
		ClientADT client;
		
		if (customerList != null) {
			for (int t=0; t<customerList.size(); t++) {
				client = (ClientADT) customerList.elementAt(t);
				address = (AddressADT) client.getAddress();
				
				values.addElement(client.getClientName());
				values.addElement(address.getStreet());
				values.addElement(address.getCity());
				values.addElement(address.getState());
				values.addElement(address.getZip());
				values.addElement(address.getCountry());
				values.addElement(address.getPhoneNumber());
			}
			
			if (values.size() == 0) {
				// Creates an "empty" vector with the data
				
				for (int t=0; t<7; t++) {
					values.addElement(new String(""));
				}				
			}
		}
		
		tabModel = new CBOSTableModel(headers, values);
		editTable.setModel(tabModel);
		
	}
	
	/**
	 * Fills the table with users data
	 */
	private void editUsers() {

		UserInterface user = this.getServerFactory().getUser();
		Vector userList = user.getUsersList(); // List of UserADT
		
		Vector headers = new Vector(0,1); // Table titles for each row
		Vector values  = new Vector(0,1); // Table values
		Vector types   = new Vector(0,1); // Table types for each row
		
		headers.addElement("Name");     // column 0
		headers.addElement("Login");    // column 1
		headers.addElement("Password"); // column 2
		headers.addElement("Function"); // column 3
		
		types.addElement(String.class); // column 0
		types.addElement(String.class); // column 1
		types.addElement(String.class); // column 2
		types.addElement(String.class); // column 3
		
		UserADT userData = null;
		UserFunctionADT function;
		
		if (userList != null) {
			for (int t=0; t<userList.size(); t++) {
				userData = (UserADT) userList.elementAt(t);
				function = (UserFunctionADT) userData.getFunction();
				values.addElement(userData.getName());
				values.addElement(userData.getLogin());
				values.addElement(userData.getPassword());
				
				if (print)
					System.out.println("DataEntryClient: Adding function "+function.toString());
				
				values.addElement(function.toString());
			}
			
			if (values.size() == 0) {
				// Creates an "empty" vector with the data
				
				for (int t=0; t<4; t++) {
					values.addElement(new String(""));
				}				
			}
		}
		
		tabModel = new CBOSTableModel(headers, values, types);
		editTable.setModel(tabModel);
		
		TableColumn qualityColumn = editTable.getColumnModel().getColumn(3);
		JComboBox qualityComboBox = new JComboBox();
		
		qualityComboBox.addItem("manager");
		qualityComboBox.addItem("buy_attendant");
		qualityComboBox.addItem("sale_attendant");
		qualityComboBox.addItem("warehouse");
		
		qualityColumn.setCellEditor(new DefaultCellEditor(qualityComboBox));
		
	}
	
	/**
	 * Fills the table with kinds data
	 */
	private void editKinds() {
		
		KindInterface kind = this.getServerFactory().getKind();
		Vector kindList = kind.getAllKinds(); // List of ClientADT
		
		Vector headers = new Vector(0,1); // Table titles
		Vector values  = new Vector(0,1); // Table values
		
		headers.addElement("Coffee Kind");
		
		if (kindList != null) {
			
			String kindName; // temp variable
			
			for (int t=0; t<kindList.size(); t++) {
				kindName = (String) kindList.elementAt(t);
				values.addElement(kindName);
			}
			
			if (kindList.size() == 0) {
				// Creates an "empty" vector with the data
				values.addElement(new String(""));
				
			}
		}
		
		tabModel = new CBOSTableModel(headers, values);
		editTable.setModel(tabModel);
		
	}
}
