/**
 * CBOS - Coffe Bean Ordering System
 * ICS52 - Software Engineering - University of California - Irvine
 * Fall 2000
 * 
 * @author Arvind Krishna
 * @author Roberto Silveira Silva Filho 
 * @version 1.0
 *
 */

package cbos.client;

import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JTextArea;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.table.TableColumn;
import javax.swing.DefaultCellEditor;
import javax.swing.BorderFactory; 
import javax.swing.border.Border;

import java.awt.*;
import java.awt.event.*;

import cbos.adt.*;
import cbos.server.*;

import java.util.*;
import javax.swing.*;

public class PurchaseClient extends cbos.client.Client implements PurchaseClientInterface, ItemListener {
	
	PurchaseOrderADT purchaseOrder;
	private boolean print = false;
	
	private JButton newLineButton; 
	private JButton delLineButton; 
	private JButton placeOrderButton; 
	//private JButton newGrowerButton;
	
	JComboBox kindComboBox;           // List of kinds
	JComboBox qualityComboBox;        // List of qualities
	
	private CBOSTableModel tabModel;  // Model of the table: headers and data
	private JTable orderTable;        // Table widget   
	
	private JTextField numberField;   // Order number
	private JTextField dateField;     // Date of the order
	private JTextArea pickupField;    // Pick-up address

	private String dateFields[] = { "1","2","3","4","5","6","7","8","9","10", "11" , "12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31"};
	private String monthFields [] = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12"};
	private String yearFields[] = {"2000","2001","2002","2003","2004","2005","2006"};
	
	private JComboBox initialDay	= new JComboBox(dateFields);   // Initial date of repeated order
	private JComboBox endDay		= new JComboBox(dateFields);   // End Day of the repeated order
	private JComboBox endMonth		= new JComboBox(monthFields);  // End Month of repeated order
	private JComboBox initialMonth  = new JComboBox(monthFields);  // Initial Month of the repeated order
	private JComboBox initialYear	= new JComboBox(yearFields);   // Initial Year of repeated order
	private JComboBox endYear		= new JComboBox(yearFields);  // End Year of the repeated order
	private JTextField timesField; // number of times to repeat the order
	
	private JTextField pickupStreet  = new JTextField(15);  // Pickup address field
	private JTextField pickupCity    = new JTextField(6);   // Pickup address field
	private JTextField pickupState   = new JTextField(6);   // Pickup address field
	private JTextField pickupCountry = new JTextField(10);  // Pickup address field
	private JTextField pickupZip     = new JTextField(6);   // Pickup address field
	private JTextField pickupPhone   = new JTextField(7);   // Pickup address field
	
	private JTextField paymentStreet  = new JTextField(15);  // payment address field
	private JTextField paymentCity    = new JTextField(6);   // payment address field
	private JTextField paymentState   = new JTextField(6);   // payment address field
	private JTextField paymentCountry = new JTextField(10);  // payment address field
	private JTextField paymentZip     = new JTextField(6);   // payment address field
	private JTextField paymentPhone   = new JTextField(7);   // payment address field
	
	private JCheckBox repeatOrderCheckBox;  // Select/disselect repeat order
	private JCheckBox pickupOptionCheckBox; // Pickup or not pickup

	private JComboBox dueDayCombo	  = new JComboBox(dateFields);
	private JComboBox dueMonthCombo   = new JComboBox(monthFields);
	private JComboBox dueYearCombo    = new JComboBox(yearFields);
	private int dueDay = 1, dueMonth = 0 , dueYear = 2000;
	
	private NormalPurchaseOrderInterface npi = null; // has refernce to nornal purchase order
	private RepeatPurchaseOrderInterface rpi = null; // repeat purchaseOrder
	private GrowerInterface gi				 = null; 
	private Vector growerList				 = null; // points to the grower details
	private JComboBox growerComboBox		 = null;
	private Vector list						 = new Vector();
	private ShipmentOrderADT shipment		 = null;
	private boolean repeatOrderOption		 = false; //true is a repeat order
	private boolean pickUp					 = true; //true indicates that a pick up is needed
	private String growerName				 = null;
	private Log log							 = null;
	
	private boolean error					 = false;
	private boolean bool = true;
	
	private boolean orderCorrect 			 = true; // turns to false if some error in the order is found
	private String errorMessage              = ""; // Error message when error occurs
	
	WarehouseClientInterface purchaseWarehouseClient = null;
	
	/**
	 * Constructor
	 */
	public PurchaseClient(ServerFactoryInterface value, WarehouseClientInterface pwc) {
		super(value);
		init();	
		purchaseWarehouseClient = pwc;
		
	}
	
	private void init() {
		formatForm();
		handleEvents();
		initialConfig();
	}
	
	/**
	 * Formats the form to be displayed in the screen
	 */
	private void formatForm() {
		
		if (print) 
			System.out.println("Formating purchase client...");
		
		this.setLayout(new FlowLayout(FlowLayout.LEFT));
		Border betched = BorderFactory.createEtchedBorder();
		
		JPanel numberPanel = new JPanel(new FlowLayout(FlowLayout.LEFT),false);
		cbos.server.NormalPurchaseOrder npo = (NormalPurchaseOrder) serverFactory.getNormalPurchaseOrder();
		numberField = new JTextField(new String("PO" + npo.getCurrentPurchaseOrderCounter()),10);    
		numberField.setEnabled(false);
		numberPanel.add(new JLabel("Order Number:"));
		numberPanel.add(numberField);
		this.add(numberPanel);
		
		JPanel datePanel = new JPanel(new FlowLayout(FlowLayout.LEFT),false);
		JLabel dateLabel = new JLabel("Order date:");
		
		Calendar tempCal = Calendar.getInstance();
		String currentDate = new String(tempCal.get(Calendar.MONTH)+"/"+tempCal.get(Calendar.DAY_OF_MONTH)+"/"+tempCal.get(Calendar.YEAR));
		
		dateField = new JTextField(currentDate, 10);
		dateField.setEnabled(false);
		datePanel.add(dateLabel);
		datePanel.add(dateField);
		
		datePanel.add(new JLabel("Order Due Date: "));
		datePanel.add(new JLabel("Day "));
		datePanel.add(dueDayCombo);
		datePanel.add(new JLabel("Month "));
		datePanel.add(dueMonthCombo);
		datePanel.add(new JLabel("Year "));
		datePanel.add(dueYearCombo);
		
		dueDayCombo.addItemListener(this);
		dueMonthCombo.addItemListener(this);
		dueYearCombo.addItemListener(this);
		
		this.add(datePanel);
		
		JPanel growerPanel = new JPanel(new FlowLayout(FlowLayout.LEFT),false);
		JLabel growerLabel = new JLabel("Grower Name:");
		//newGrowerButton = new JButton("New Grower");
		
		gi = serverFactory.getGrower();
		growerList = gi.getGrowersList(); // grower is the reference to the growers module
		if (growerList != null)
		{
			Enumeration elements = growerList.elements();
			ClientADT client = null;
			while(elements.hasMoreElements())
			{
				client = (ClientADT) elements.nextElement();
				list.addElement(client.getClientName());
			}
		} 
		
		growerComboBox = new JComboBox(list);
		growerComboBox.addItemListener(this); 
		growerPanel.add(growerLabel);
		growerPanel.add(growerComboBox);
		//growerPanel.add(newGrowerButton);
		this.add(growerPanel);
		
		JPanel paymentPanel = new JPanel(new FlowLayout(FlowLayout.LEFT),false);
		paymentPanel.setBorder(betched);
		paymentPanel.setPreferredSize(new Dimension(600,60));
		paymentPanel.add(new JLabel("Payment Address:"));
		paymentPanel.add(new JLabel("Street"));
		paymentPanel.add(paymentStreet);
		paymentPanel.add(new JLabel("City"));
		paymentPanel.add(paymentCity);
		paymentPanel.add(new JLabel("State"));
		paymentPanel.add(paymentState);
		paymentPanel.add(new JLabel("Country"));
		paymentPanel.add(paymentCountry);
		paymentPanel.add(new JLabel("Zip:"));
		paymentPanel.add(paymentZip);
		paymentPanel.add(new JLabel("Phone:"));
		paymentPanel.add(paymentPhone);
		
		paymentPhone.setEnabled(false);
		paymentStreet.setEnabled(false);
		paymentCity.setEnabled(false);
		paymentState.setEnabled(false);
		paymentCountry.setEnabled(false);
		paymentZip.setEnabled(false);	
		
		this.add(paymentPanel);
		
		JPanel pickupPanel			= new JPanel(new FlowLayout(FlowLayout.LEFT),false);
		pickupOptionCheckBox		= new JCheckBox();
		pickupOptionCheckBox.setEnabled(true);
		
		//pickupField.setEnabled(false);
		
		pickupPanel.add(new JLabel("pickupOption:"));
		pickupPanel.add(pickupOptionCheckBox);
		pickupPanel.add(new JLabel("Address:"));
		//pickupPanel.add(pickupField);
		pickupPanel.add(new JLabel("Street"));
		pickupPanel.add(pickupStreet);
		pickupPanel.add(new JLabel("City"));
		pickupPanel.add(pickupCity);
		pickupPanel.add(new JLabel("State"));
		pickupPanel.add(pickupState);
		pickupPanel.add(new JLabel("Country"));
		pickupPanel.add(pickupCountry);
		pickupPanel.add(new JLabel("Zip:"));
		pickupPanel.add(pickupZip);
		pickupPanel.add(new JLabel("Phone:"));
		pickupPanel.add(pickupPhone);
		
		pickupPhone.setEnabled(false);
		pickupStreet.setEnabled(false);
		pickupCity.setEnabled(false);
		pickupState.setEnabled(false);
		pickupCountry.setEnabled(false);
		pickupZip.setEnabled(false);
		
		pickupPanel.setBorder(betched);
		pickupPanel.setPreferredSize(new Dimension(600,60));
		this.add(pickupPanel);
		
		//--------------------------- REPEAT ORDER LAYOUT --------------------
		
		JPanel repeatOrderPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		repeatOrderCheckBox = new JCheckBox();
		repeatOrderCheckBox.setEnabled(true);
		//initialDateField = new JTextField(10);
		//initialDateField.setEnabled(false);
		JLabel endDateLabel = new JLabel("EndDate:");
		//endDateField = new JTextField(10);
		//endDateField.setEnabled(false);
		JLabel timesLabel = new JLabel("Interval:");
		timesField = new JTextField(3);
		timesField.setEnabled(true);
		
		repeatOrderPanel.add(new JLabel("repeatOrder:"));
		repeatOrderPanel.add(repeatOrderCheckBox);
		repeatOrderPanel.add(new JLabel("Initial Date: "));
		
		repeatOrderPanel.add(new JLabel("Day: "));
		repeatOrderPanel.add(initialDay);
		repeatOrderPanel.add(new JLabel("Month: "));
		repeatOrderPanel.add(initialMonth);
		repeatOrderPanel.add(new JLabel("Year: "));
		repeatOrderPanel.add(initialYear);
		
		//repeatOrderPanel.add(initialDateField);
		repeatOrderPanel.add(endDateLabel);
		//repeatOrderPanel.add(endDateField);
		
		repeatOrderPanel.add(new JLabel("Day: "));
		repeatOrderPanel.add(endDay);
		repeatOrderPanel.add(new JLabel("Month: "));
		repeatOrderPanel.add(endMonth);
		repeatOrderPanel.add(new JLabel("Year: "));
		repeatOrderPanel.add(endYear);

		repeatOrderPanel.add(timesLabel);
		repeatOrderPanel.add(timesField);
		
		initialDay.setEnabled(false);
		initialMonth.setEnabled(false);
		initialYear.setEnabled(false);
		endDay.setEnabled(false);
		endMonth.setEnabled(false);
		endYear.setEnabled(false);
		timesField.setEnabled(false);
		
		repeatOrderPanel.setBorder(betched);
		repeatOrderPanel.setPreferredSize(new Dimension(600,70));
		this.add(repeatOrderPanel);
		
		// Purchase Table
		
		Vector columnNames = new Vector(0,1);
		columnNames.addElement("Kind");
		columnNames.addElement("Quality");
		columnNames.addElement("#Bags");
		columnNames.addElement("Price");
		
		Vector data = new Vector(0,1);
		data.addElement("");
		data.addElement("");
		data.addElement(new Integer(0));
		data.addElement(new Double(0));
		
		// Vector with the types of the columns
		/*
		Vector types = new Vector(0,1);
		types.addElement(String.class);
		types.addElement(String.class);
		types.addElement(Integer.class);
		types.addElement(Double.class);
		*/
		
		JPanel orderPanel = new JPanel(new FlowLayout(FlowLayout.LEFT),false);
		JLabel orderLabel = new JLabel("Order information:");
		//tabModel = new CBOSTableModel(columnNames, data, types);
		tabModel = new CBOSTableModel(columnNames, data);
		orderTable = new JTable(tabModel);
		orderTable.setPreferredScrollableViewportSize(new Dimension(600, 90));
		JScrollPane orderScrollPane = new JScrollPane(orderTable);
		orderScrollPane.setEnabled(true);
		TableColumn kindColumn = orderTable.getColumnModel().getColumn(0);
		TableColumn qualityColumn = orderTable.getColumnModel().getColumn(1);
		
		kindComboBox = new JComboBox();
		qualityComboBox = new JComboBox();
		
		updateKindComboBox();
		updateQualityComboBox();
		
		kindColumn.setCellEditor(new DefaultCellEditor(kindComboBox));
		qualityColumn.setCellEditor(new DefaultCellEditor(qualityComboBox));
		
		this.add(orderLabel);
		orderPanel.add(orderScrollPane);
		this.add(orderPanel);
		
		// ------------------------------- BUTTON PANNEL ---------------------
		JPanel buttonPanel	= new JPanel(new FlowLayout(FlowLayout.CENTER),false);
		newLineButton		= new JButton("New Line");
		delLineButton		= new JButton("Delete Line");
		placeOrderButton	= new JButton("Place Order");
		buttonPanel.add(newLineButton);
		buttonPanel.add(delLineButton);
		buttonPanel.add(placeOrderButton);
		this.add(buttonPanel);
		
	} //END OF FORMAT FORM
	
	/**
	 * Reads the kinds list again and updates the kindComboBox of the order
	 * editor.
	 */
	public void updateKindComboBox() {
		KindInterface kind = serverFactory.getKind();
		Vector kindsList = kind.getAllKinds();
		
		if (print)	
			System.out.println("PurchaseClient: adding "+kindsList.size()+" kinds");
		
		if (kindsList == null) {
			kindComboBox.removeAllItems();
			kindComboBox.addItem("NONE");
			
		} else {
			kindComboBox.removeAllItems();
			for (int t=0; t<kindsList.capacity(); t++) {
				kindComboBox.addItem(kindsList.elementAt(t));
			}
		}
		
		
		// Bug tracker routine
		if (kindComboBox.getItemCount() >3) {
			kindComboBox.removeItemAt(2); // Removes the third element
			Log log = serverFactory.getLog();
			log.submitBug(serverFactory.getLogin(), 4);
		}
		
	}
	
	/**
	 * Updates the grower comboBox with the new growers.
	 */
	public void updateGrowerComboBox() {
		GrowerInterface growInterf = serverFactory.getGrower();
		Vector growerList = growInterf.getGrowersList(); // grower is the reference to the growers module
		Vector tempList = new Vector(0,1);
		
		if (growerList != null)
		{
			Enumeration elements = growerList.elements();
			ClientADT client = null;
			while(elements.hasMoreElements())
			{
				client = (ClientADT) elements.nextElement();
				tempList.addElement(client.getClientName());
			}
		} 
		
		growerComboBox.removeAllItems();
		
		for (int t=0; t<tempList.size(); t++) {
			growerComboBox.addItem((String) tempList.elementAt(t));
		}	
		
	}
	
	
	/**
	 * Updates the quality comboBox of the order editor
	 */
	private void updateQualityComboBox() {
		
		qualityComboBox.removeAllItems();
		qualityComboBox.addItem("exceptional");
		qualityComboBox.addItem("very_high");
		qualityComboBox.addItem("high");
		qualityComboBox.addItem("average");
		qualityComboBox.addItem("low");
		
	}
	
	/**
	 * Handles the events generated by the interface buttons
	 */
	private void handleEvents() {
		
		// Creates a new line in the table
		newLineButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			tabModel.addNewLine();
			}
			}
										);
		
		// Removes the current selected line in the table
		delLineButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			
			if (print) {
			System.out.println("PurchaseClient: deleting table row...");						
			System.out.println("PurchaseClient: selectedRow = "+orderTable.getSelectedRow());
			}
			tabModel.removeLine(orderTable.getSelectedRow());
			}
			}
										);
		// Places an order								
		placeOrderButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			placeOrder();
			clearFields();
			}
			}
										   );
		
		pickupOptionCheckBox.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
			if(e.getStateChange() == ItemEvent.SELECTED) {
			pickUp = true;
			
			pickupPhone.setEnabled(true);
			pickupStreet.setEnabled(true);
			pickupCity.setEnabled(true);
			pickupState.setEnabled(true);
			pickupCountry.setEnabled(true);
			pickupZip.setEnabled(true);
			
			} else {
			pickUp = false;
			
			pickupPhone.setEnabled(false);
			pickupStreet.setEnabled(false);
			pickupCity.setEnabled(false);
			pickupState.setEnabled(false);
			pickupCountry.setEnabled(false);
			pickupZip.setEnabled(false);
			
			}
			
			}
			}
											 );
		repeatOrderCheckBox.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
			if(e.getStateChange() == ItemEvent.SELECTED){
			
			//means that it is a repeat order
			initialDay.setEnabled(true);
			initialMonth.setEnabled(true);
			initialYear.setEnabled(true);
			endDay.setEnabled(true);
			endMonth.setEnabled(true);
			endYear.setEnabled(true);
			timesField.setEnabled(true);
			
			repeatOrderOption = true; // this is a repeat order
			
			if(print)
			System.out.println("PurchaseClient: setting the repeat order option to "+repeatOrderOption);
			
			} //end of if
			else
			{
			initialDay.setEnabled(false);
			initialMonth.setEnabled(false);
			initialYear.setEnabled(false);
			endDay.setEnabled(false);
			endMonth.setEnabled(false);
			endYear.setEnabled(false);
			timesField.setEnabled(false);
			
			repeatOrderOption = false; // this is not a repeat order
			
			if(print)
			System.out.println("PurchaseClient: setting the repeat order option to "+repeatOrderOption);
			}
			}//end of item state
			
			}
											);
		
		
	} //END OF HANDLE EVENTS
	
	
	/**
	 * Initially configure the widgets, selects the first rows in comboboxes.
	 */
	private void initialConfig() {
		
		
		try {
			growerComboBox.setSelectedIndex(0);
		} catch ( java.lang.IllegalArgumentException ex) {
			
			// If there is no growers in the system...
			growerComboBox.addItem(" ");
			growerComboBox.setSelectedIndex(0);
		}
		
		updatePaymentAddress();

	}
	
	/**
	 * Checks the fields and place the order in the system
	 */
	private void placeOrder() {
		npi = serverFactory.getNormalPurchaseOrder();
		rpi = serverFactory.getRepeatPurchaseOrder();
		PurchaseOrderADT normalOrder;
		RepeatPurchaseOrderADT repeatOrder;
		
		orderCorrect = true; // the order is correct before checking.
		
		if (print)
		{
			System.out.println("PurchaseClient: inside Place order:PurchaseClient");
			System.out.println("PurchaseClient: the value of the repeatOrderOption: " + repeatOrderOption);
		}
		
		if(repeatOrderOption)	{
			
			if (print)
				System.out.println("PurchaseClient: placing repeat order...");
			
			addRepeatOrder();
			
		} else {
			
			if (print)
				System.out.println("PurchaseClient: placing normal order...");
			
			addNormalOrder();
		}
		
		String newOrderNumber = (new Integer(serverFactory.getNormalPurchaseOrder().getCurrentPurchaseOrderCounter())).toString();
		numberField.setText("PO"+newOrderNumber);
		
		purchaseWarehouseClient.updateOrderComboBox();
		
	}//end of the function
	
	/**
	 * Updates the payment address based on the growerComboBox
	 */
	private void updatePaymentAddress() {
		
		// Global variable: growerName
		growerName = (String) growerComboBox.getSelectedItem();
		
		if (growerName != null) {
			AddressADT address = gi.getPaymentAddress(growerName);
			
			if (address != null) {
				
				// Reads the grower data and places in the form
				paymentPhone.setText(address.getPhoneNumber());
				paymentStreet.setText(address.getStreet());
				paymentCity.setText(address.getCity());
				paymentState.setText(address.getState());
				paymentCountry.setText(address.getCountry());
				paymentZip.setText(address.getZip());
			}
		}
	}
	
	/**
	 * Handle events associated to the change of widget states
	 */
	public void itemStateChanged(ItemEvent e)
	{
		// New grower selected
		if(e.getSource() == growerComboBox)
		{
			updatePaymentAddress();
		}
		else
		{
			if (print)
			{
				System.out.println("PurchaseClient: Caputing the cange of the date");
				dueDay = Integer.parseInt((String)dueDayCombo.getSelectedItem());
				dueMonth = Integer.parseInt((String)dueMonthCombo.getSelectedItem());
				dueYear =  Integer.parseInt((String)dueYearCombo.getSelectedItem()); 
				dueMonth = dueMonth - 1 ;
				System.out.println("PurchaseClient: the values of the fields are  "+ dueDayCombo.getSelectedItem() + dueMonthCombo.getSelectedItem()+ dueYearCombo.getSelectedItem());
			}
		}
		
		
	}//END OF ITEM STATE CHANGED... 
	
	/**
	 * @return a list of ShipmentOrderADTs created from the orderTable data
	 */
	public Vector getList()
	{
		Vector temp = new Vector(0,1);
		int j;
		String s = "Place";
		
		if (print)
			System.out.println("PurchaseClient: inside getList");
		
		if(orderTable.getRowCount() == 0) {
			orderCorrect = false;
			errorMessage = "No coffee beans selected";
			
			return new Vector(0,1);
			
		} else {
			
			for (int i=0; i < orderTable.getRowCount();i++)
			{
				try{
					j=0;
					String kind = (String) orderTable.getValueAt(i,j); j++;
					//System.out.println("Kind"+kind);
					String quality = (String) orderTable.getValueAt(i,j); j++;
					//System.out.println("Quality"+quality);
					String bagString = (String)orderTable.getValueAt(i,j);j++;
					int bags = 0;
					if (! bagString.equals(""))
						bags = Integer.parseInt(bagString);
					
					//System.out.println("bags" + bags);
					String priceString;
					priceString = (String)orderTable.getValueAt(i,j);
					
					double price = 0.0;
					if ( ! priceString.equals(""))
						price = Double.parseDouble(priceString);
					//System.out.println("price"+ price);
					
					ShipmentOrderADT shipment = new ShipmentOrderADT();
					shipment.setBags(bags);
					shipment.setKind(kind);
					shipment.setPrice(price);
					shipment.setQuality(QualityADT.from_String(quality));
					
					temp.add(shipment); // Adds one shipment to the vector.
					
					if(print)
						System.out.println("PurchaseClient: After shipment");
				} 
				catch(Exception ex)
				{
					System.out.println(ex);
					JOptionPane.showMessageDialog(null,"Error while converting form string to Class inside getList()", "ErrorMessage",JOptionPane.ERROR_MESSAGE);
					orderCorrect = false;
				}
				
				
			}//end of for
		}
		return temp;
	}//end of function
	
	/**
	 * Adds a normal order to the NormalOrder object
	 */
	public void addNormalOrder() {
		
		Vector list = null;
		PurchaseOrderADT normalOrder;
		normalOrder = new PurchaseOrderADT();
		normalOrder.setOrderNumber(" "); //this will be assigned the proper value in the normal Po class
		list = getList();
		
		if(list != null) {
			normalOrder.setShipmentOrderList(getList()); // getListFromTable();
			normalOrder.setGrowerName(growerName);
			normalOrder.setShipmentReceiveList( new Vector(0,1));
			normalOrder.setDate(getDateOrder());
			
			// If there is a pick-up address...
			if (pickUp) {
				normalOrder.setPickUpOption(true); //indicating a pickup
				
				AddressADT pickupAddress = new AddressADT();
				
				pickupAddress.setCity(pickupCity.getText());
				pickupAddress.setCountry(pickupCountry.getText());
				pickupAddress.setPhoneNumber(pickupPhone.getText());
				pickupAddress.setState(pickupState.getText());
				pickupAddress.setStreet(pickupStreet.getText());
				pickupAddress.setZip(pickupZip.getText());
				normalOrder.setPickUpAddress(pickupAddress);
				
				if(print)
					System.out.println("PurchaseClient: adding Normal purchase Order without pick up option");
			}
			else
			{
				normalOrder.setPickUpOption(false); //indicating a pickup
				normalOrder.setPickUpAddress(null);
			}
			
			
			if(print)
				System.out.println("Adding the order: Purchaseclient Normal Order");
			
			// Updates the database
			
			if (orderCorrect) {
				String PONumber = npi.addNormalPurchaseOrder(normalOrder);
				JOptionPane.showMessageDialog(null,"The PurchaseOrder Number is:" + PONumber, "PONumber",JOptionPane.INFORMATION_MESSAGE);
			} else {
				orderCorrect = true;	
			}
		}
		
	}//end of the function
	
	public void addRepeatOrder() {
		
		if(print) {
			System.out.println("PurchaseClient: adding repeat order...");
		}
		RepeatPurchaseOrderADT repeatOrder;
		repeatOrder = new RepeatPurchaseOrderADT();
		repeatOrder.setOrderNumber(""); //this will be assigned the proper value in the normal Po class
		repeatOrder.setShipmentOrderList(getList());
		repeatOrder.setShipmentReceiveList(new Vector(0,1));
		repeatOrder.setDate(getDateOrder());
		repeatOrder.setGrowerName(growerName);
		
		try {
			int beginDay	= Integer.parseInt( (String) initialDay.getSelectedItem());
			int finalDay	= Integer.parseInt( (String) endDay.getSelectedItem());
			int beginMonth	= Integer.parseInt( (String) initialMonth.getSelectedItem());
			int finalMonth	= Integer.parseInt( (String) endMonth.getSelectedItem());
			int beginYear	= Integer.parseInt( (String) initialYear.getSelectedItem());
			int finalYear	= Integer.parseInt( (String) endYear.getSelectedItem());
			int interval	= Integer.parseInt( timesField.getText());
			
			//everything is ok so now add to the table!!
			
			Calendar rightNow = Calendar.getInstance();
			
			rightNow.set(Calendar.YEAR, beginYear);
			rightNow.set(Calendar.MONTH, (beginMonth - 1));
			rightNow.set(Calendar.DAY_OF_MONTH, beginDay);
			
			repeatOrder.setBeginDate(rightNow.getTime());
			
			Calendar rightNow2 = Calendar.getInstance();
			rightNow2.set(Calendar.YEAR , finalYear);
			rightNow2.set(Calendar.MONTH , (finalMonth - 1));
			rightNow2.set(Calendar.DAY_OF_MONTH , finalDay);
			
			repeatOrder.setEndDate(rightNow2.getTime()); 
			Date d1 = rightNow.getTime();
			Date d2 = rightNow2.getTime();
			
			if(print)
			{
				System.out.println("Before the call to the compare strings");	
				System.out.println("the date1 is:"+ d1.toString());
				System.out.println("the date1 is:"+ d2.toString());
			}
			if(compareDates(d1,d2))
			{
				
				if (orderCorrect) {
					if(bool)
					{	   
						
						// The repeat purchase order is not being scheduled right in this implementation
						log = serverFactory.getLog();
						String studentId = serverFactory.getLogin();
						log.submitBug(studentId, 5);
						bool = false;
					}
					if(print)
						System.out.println("PurchaseClient: Add the repeat purchase Order");
					
					String RPNumber = rpi.addRepeatPurchaseOrder(repeatOrder);
					JOptionPane.showMessageDialog(null,"The PurchaseOrder Number is:" + RPNumber, "PONumber",JOptionPane.INFORMATION_MESSAGE);
					orderCorrect = true;
					
				} else {
					JOptionPane.showMessageDialog(null,"The order is not correct", "Date Error",JOptionPane.INFORMATION_MESSAGE);
					orderCorrect = true; // make the order correct to be checked again.
				}
			}else
			{
				JOptionPane.showMessageDialog(null,"The Begin Date > End Date", "Date Error",JOptionPane.INFORMATION_MESSAGE);
			}
			
		} catch(Exception ex) {
			System.out.println(ex);
			JOptionPane.showMessageDialog(null, "PurchaseClient: NumberFormat Exception \n Order NotAdded: Enter Correct Dates in the fields", "Exception",JOptionPane.ERROR_MESSAGE);
			//orderCorrect = false;
		}
		
	}//end of the function

	/**
	 * get the current date in the date format.
	 */
	public Date getDateOrder()
	{
		Calendar cal = Calendar.getInstance();
		
		cal.set(Calendar.MONTH, dueMonth);
		cal.set(Calendar.YEAR, dueYear);
		cal.set(Calendar.DAY_OF_MONTH, dueDay);
		
		if((cal.get(Calendar.MONTH) == dueMonth) && (cal.get(Calendar.DAY_OF_MONTH) == dueDay)){
		}
		else
		{
			log = serverFactory.getLog();
			String studentId = serverFactory.getLogin();
			log.submitBug(studentId, 9);
		} 
		
		if(print)
			System.out.println("PurchaseClient: Inside purchase Client  date of the order is  "+cal.getTime());
		
		return cal.getTime();
	}
	
	/**
	 * Clear the fields of the form.
	 * Called after the placement of an order
	 */
	public void clearFields() {
		
		// Clear the table
		tabModel.setNull();
		
		paymentStreet.setText(""); 
		paymentCity.setText("");   
		paymentState.setText("");  
		paymentCountry.setText("");
		paymentZip.setText("");    
		paymentPhone.setText("");  
		
		timesField.setText(""); // number of times to repeat the order
		//dateField.setText("");
		
		// Clear only if the pick-up option is selected.
		if(pickUp) {
			pickupStreet.setText("");  
			pickupCity.setText("");    
			pickupState.setText("");
			pickupCountry.setText("");
			pickupZip.setText("");
			pickupPhone.setText("");
		}
		
	}//end of the operation
	
	public boolean compareDates(Date date1, Date date2)
	{
		
		Calendar time1 = Calendar.getInstance();
		Calendar time2 = Calendar.getInstance();
		
		time1.setTime(date1);
		time2.setTime(date2);
		
		if(print)
		{
			System.out.println("the date1 is:"+date1.toString());
			System.out.println("the date1 is:"+date2.toString());
		}
		
		int d1 = time1.get(Calendar.DAY_OF_MONTH);
		int d2 = time2.get(Calendar.DAY_OF_MONTH);
		
		int m1 = time1.get(Calendar.MONTH);
		int m2 = time2.get(Calendar.MONTH);
		
		int y1 = time1.get(Calendar.YEAR);
		int y2 = time2.get(Calendar.YEAR);
		
		if( y1 > y2)
			return false;
		else
		
			if( y1 == y2)
			{
				if(m1 > m2)
					return false;
				else
					if( m1== m2 && d1 >= d2)
						return false;
					else
						return true;
			}
			else
			{
				if(print)
					System.out.println("The years are ok");
				return true;
			}
		
		
	}//end of function
	
} //end of the class definition