/**
 * CBOS - Coffe Bean Ordering System
 * ICS52 - Software Engineering - University of California - Irvine
 * Fall 2000
 * 
 * @author Arvind Krishna
 * @author Roberto Silveira Silva Filho 
 * @version 1.0
 *
 */

package cbos.client;

import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.swing.JEditorPane;
import javax.swing.JTextPane;
import javax.swing.JScrollPane;
import javax.swing.JComboBox;
import java.awt.*;
import java.awt.event.*;



import java.util.*;
import cbos.adt.*;
import cbos.server.*;
import cbos.client.reports.*;

public class ReportsClient extends cbos.client.Client implements ReportsInterface, ActionListener, ItemListener {

	private DailyActualPickupInterface actualPickUp;
	private DailyPickupInterface scheduledPickup;
	private PurchasePerGrowerInterface purchaseGrower;
	private DailyPaymentsInterface dailyPayments;
	private DailyScheduleShipmentsInterface scheduledShipments;
	private Log log;
	private String studentId;
	
	private GrowerInterface gi;
	private JTextArea reportText ;
	private JButton dailyPickupButton ;
	private JButton dailyActualPickupsButton ;
	private JButton dailyPaymentsButton ;		
	private JButton dailyShedShipButton ;
	private JButton checkInventoryButton;
	private JButton monthInventoryButton;
	private JButton monthSalesButton ;
	
	private JComboBox growerComboBox ; // List of growers
	private JComboBox customerComboBox; // List of customers
	private String growerName = null;
	private String customerName = null;
	private JButton monthPurchaseButton ;
	private Vector list = new Vector(); // has the list of the growers, only the names
	private Vector growerList = null; // points to the grower details
	//private JTextField growerField = new JTextField(10);
	
	private InventoryClient inventoryInstance; // Inventory window reference
	
	private boolean print = false;

	/**
	 * Constructor
	 */
	public ReportsClient(ServerFactoryInterface value) {
		super(value);
		formatForm();
		createReports();
		handleEvents();
	}

	private void createReports()
	{
		
		actualPickUp = new DailyActualPickup(serverFactory.getNormalPurchaseOrder());
		scheduledPickup = new DailyPickupSchedule(serverFactory.getNormalPurchaseOrder());
		purchaseGrower = new PurchasePerGrower(serverFactory.getNormalPurchaseOrder(), serverFactory.getGrower());
		dailyPayments = new DailyPayments(serverFactory.getNormalPurchaseOrder(), serverFactory);
		scheduledShipments = new DailyScheduleShipments(serverFactory.getNormalSalesOrder());
		log = serverFactory.getLog();
		studentId = serverFactory.getLogin();
	}
	
	private void formatForm() {
		
		this.setLayout(new FlowLayout(FlowLayout.LEFT));
		
		JPanel dailyButtonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT),false);
		dailyPickupButton = new JButton("Daily Pick-up Sched");
		dailyActualPickupsButton = new JButton("Daily Actual Pick-ups");
		dailyPaymentsButton = new JButton("Daily Payments");		
		dailyShedShipButton = new JButton("Daily Sched Ships");
		checkInventoryButton= new JButton("Inventory");
		
		ClientADT client = null;
		String growerName = null;
		
		dailyButtonPanel.add(dailyPickupButton);
		dailyButtonPanel.add(dailyActualPickupsButton);
		dailyButtonPanel.add(dailyPaymentsButton);
		dailyButtonPanel.add(dailyShedShipButton);
				
		this.add(dailyButtonPanel);
		
		//adding the listener interfaces
		dailyPickupButton.addActionListener(this);
		dailyActualPickupsButton.addActionListener(this);
		dailyPaymentsButton.addActionListener(this);
		dailyShedShipButton.addActionListener(this);
		
		JPanel monthButtonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT),false);
		monthInventoryButton = new JButton("Month Inventory");		
		
		monthInventoryButton.addActionListener(this);
		monthButtonPanel.add(monthInventoryButton);
		this.add(monthButtonPanel);
		
		JPanel monthPurchasePanel = new JPanel(new FlowLayout(FlowLayout.LEFT),false);
		monthPurchaseButton = new JButton("Monthly Purchase");
		JLabel growerLabel = new JLabel("Grower:");
		
		/* inorder to asssign values to the combo box: call getGrowerList(): returns
		a vector and get the grower names form the list and put it in the combo box!
		*/
		
		gi = serverFactory.getGrower();
		growerList = gi.getGrowersList(); // grower is the reference to the growers module
		list = new Vector(0,1);
		if(growerList != null)
		{
			Enumeration elements = growerList.elements();
			while(elements.hasMoreElements())
			{
				client = (ClientADT) elements.nextElement();
				list.addElement(client.getClientName());
			}
		}
		
		growerComboBox = new JComboBox(list);
		growerComboBox.addItemListener(this); 
		
		monthPurchasePanel.add(growerLabel);		
		monthPurchasePanel.add(growerComboBox);
		//monthPurchasePanel.add(growerField);
		//growerField.addActionListener(this);
		monthPurchasePanel.add(monthPurchaseButton);
		this.add(monthPurchasePanel);
		monthPurchaseButton.addActionListener(this);
		
		JPanel monthSalesPanel = new JPanel(new FlowLayout(FlowLayout.LEFT),false);
		monthSalesButton = new JButton("Monthly Sales");
		
		monthSalesButton.addActionListener(this);
		
		monthSalesPanel.add(new JLabel("Customer:"));	
		
		// Copy customer names from the customer repository
		CustomerInterface gi = serverFactory.getCustomer();
		Vector CustomerList = gi.getCustomersList(); // Customer is the reference to the Customers module
		list = new Vector(0,1);
		if (CustomerList != null)
		{
			Enumeration elements = CustomerList.elements();
			ClientADT customer = null;
			while(elements.hasMoreElements())
			{
				customer = (ClientADT) elements.nextElement();
				list.addElement(customer.getClientName());
			}
		} 
		
		customerComboBox = new JComboBox(list);
		customerComboBox.addItemListener(this); 
		
		
		this.add(checkInventoryButton);
		
		monthSalesPanel.add(customerComboBox);		
		monthSalesPanel.add(monthSalesButton);
		this.add(monthSalesPanel);
		
		//JPanel reportTextPanel = new JPanel(new FlowLayout(FlowLayout.CENTER),false);
		reportText = new JTextArea( );
		reportText.setWrapStyleWord(true);
		JScrollPane reportScrollPane = new JScrollPane(reportText);
		reportScrollPane.setPreferredSize(new Dimension(600, 350));
		reportScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		reportText.setEnabled(false);
		this.add(reportScrollPane);
		
		
		
	}// end of the function
	
	
	public void handleEvents() {

		checkInventoryButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			showInventory();
			}
			}
											  );

	
	}
	
	
	public void actionPerformed(ActionEvent e)
	{ 
		if(e.getSource() == dailyPickupButton) {
			if(print)
				System.out.println("ReportsClient: dailyPickupButton pressed! ");
			reportText.removeAll();
			reportText.setText(scheduledPickup.getOutput());	
			
		} else if(e.getSource() == dailyActualPickupsButton) {
			if(print)
				System.out.println("ReportsClient: dailyActualPickupsButton pressed! ");
			
			reportText.removeAll();
			reportText.setText(actualPickUp.getOutput());
			
		} else if(e.getSource() == dailyPaymentsButton) {
			if(print)
				System.out.println("ReportsClient: dailyPaymentsButton pressed! ");
			
			reportText.removeAll();
			reportText.setText(dailyPayments.getOutput());
			
		} else if(e.getSource() == monthPurchaseButton) {
			if(print)
				System.out.println("ReportsClient: monthPurchaseButton pressed! ");
			
			/*reportText.removeAll();
			reportText.setText(purchaseGrower.getOutput(growerName)); */
			String error = "There are no grower purchases scheduled for this month";
			reportText.setText(error);
			error = " ";
			log.submitBug(studentId, 1);
			
		} else if(e.getSource() == dailyShedShipButton) {
						
			if(print)
				System.out.println("ReportsClient: dailyShedShipButton pressed! ");
			
			reportText.removeAll();
			reportText.setText(scheduledShipments.getOutput());
			
		}else if(e.getSource() == monthInventoryButton ) {
						
			log.submitBug(studentId, 0);
			String smileFace = " ";
			for(int i=0; i <5; i++)
			smileFace += " : - )";
			reportText.removeAll();
			reportText.setText(smileFace);
			
		}else if(e.getSource() == monthSalesButton) {
						
			log.submitBug(studentId, 2);
			String error = "Fatal Exception Occured";
			reportText.setText(error);
			error = " ";
		}
		
		
	}//end of handle events 

	
	public void itemStateChanged(ItemEvent e)
	{ 
		growerName = (String) growerComboBox.getSelectedItem();
		customerName = (String) customerComboBox.getSelectedItem();
		
	}
	
	
	/**
	 * Updates the grower comboBox with the new growers.
	 */
	public void updateGrowerComboBox() {
		GrowerInterface growInterf = serverFactory.getGrower();
		Vector growerList = growInterf.getGrowersList(); // grower is the reference to the growers module
		Vector tempList = new Vector(0,1);
		
		if (growerList != null)
		{
			Enumeration elements = growerList.elements();
			ClientADT client = null;
			while(elements.hasMoreElements())
			{
				client = (ClientADT) elements.nextElement();
				tempList.addElement(client.getClientName());
			}
		} 
		
		growerComboBox.removeAllItems();
		
		for (int t=0; t<tempList.size(); t++) {
			growerComboBox.addItem((String) tempList.elementAt(t));
		}	
	
	}
	
	/**
	 * Updates the customer comboBox with the new customers.
	 */
	public void updateCustomerComboBox() {
		CustomerInterface growInterf = serverFactory.getCustomer();
		Vector customerList = growInterf.getCustomersList(); // customer is the reference to the customers module
		Vector tempList = new Vector(0,1);
		
		if (customerList != null)
		{
			Enumeration elements = customerList.elements();
			ClientADT client = null;
			while(elements.hasMoreElements())
			{
				client = (ClientADT) elements.nextElement();
				tempList.addElement(client.getClientName());
			}
		} 
		
		customerComboBox.removeAllItems();
		
		for (int t=0; t<tempList.size(); t++) {
			customerComboBox.addItem((String) tempList.elementAt(t));
		}	
	
	}
	
	/**
	 * Shows the inventory in a separate window
	 */
	private void showInventory() {
		JFrame frame = new JFrame("Inventory List");
		inventoryInstance = new InventoryClient(this.getServerFactory());
		
		frame.getContentPane().add(inventoryInstance,BorderLayout.CENTER);
		frame.setSize(460,430);
		frame.setVisible(true);
	}

	
}// end of the class


// got to press enter for the text field to listen to the event ie the grower name!!!
