/**
 * CBOS - Coffe Bean Ordering System
 * ICS52 - Software Engineering - University of California - Irvine
 * Fall 2000
 * 
 * @author Arvind Krishna
 * @author Roberto Silveira Silva Filho 
 * @version 1.0
 *
 */

package cbos.server;

public class BugTrack implements java.io.Serializable {
	
	private String key; // Autentication key
	private String studentId;
	private boolean[] bugsArray; // true = found the bug; false = do not.
	private DES des; // Algoritm to generate the key
	
	private boolean print = false;
	
	/**
	 * Constructor
	 * @param id the student id
	 * @param numberBugs the number of bugs to track
	 */
	public BugTrack(String id, int numberBugs) {
		
		studentId = id;
		
		// Creates and configures the encriptor with the appropriate parameters
		if (des == null) {
			
			int step = id.length() / 4;
			
			String alfa  = id.substring(0,step);
			String beta  = id.substring(step,2*step);
			String gamma = id.substring(2*step,3*step);
			String delta = id.substring(3*step,id.length());

			// Creates an instance of the encript algorithm with the 
			// parameters related to the uid.
			des = new DES((new Integer(alfa)).intValue(),
						  (new Integer(beta)).intValue(),
						  (new Integer(gamma)).intValue(),
						  (new Integer(delta)).intValue()
						  );
		}
		
		key = generateKey(id);
		bugsArray = new boolean[numberBugs];
		
		if (print)
			System.out.println("BugTrack: generating "+numberBugs+" bugs array...");
		
		// Initialize but array to false. No bugs up to now...
		for (int t=0; t<bugsArray.length; t++) {
			bugsArray[t] = false;
		}
	}
	
	/**
	 * Lists the bugs array in the screen
	 */
	public boolean[] listBugsArray() {
		
		System.out.print("BugTracker: bugsArray: ");
		for (int t=0; t<bugsArray.length; t++) {
			System.out.print(bugsArray[t]+", ");
		}
		System.out.println();
		
		return bugsArray;
	}
	
	/**
	 * Generate a key provided the id
	 * @param id the student id, used as the base of the key
	 */
	private String generateKey(String id) {
		
		String newKey = new String("");
		int desOutput;
		char desChar;
		
		
		for (int t=0; t<id.length(); t++) {
			String character = id.substring(t,t+1);
			
			desOutput = des.function( (new Integer(character) ).intValue());
			desChar = (char) desOutput;
			newKey = newKey.concat( (new Character(desChar)).toString() );
			
		}
		
		return newKey;
		
	}
	
	/**
	 * Submmits a bug to this file, validades the id provided.
	 * @param id the student id to be validated
	 * @param number the number of the bug submitted.
	 */
	public boolean submitBug(String id, int number) {
		
		boolean submitted = false;
		
		if ( number < bugsArray.length) {
			if ( validate(id) ) {
				bugsArray[number] = true;
				submitted = true;
				
			}
			
			if (! submitted) {
				System.out.println("BugTracker: INVALID ID for this file!!!");
			}
		} else {
			System.out.println("BugTracker: INVALID Bug number: "+number+" !!!");
		}
		return submitted;
	}
	
	
	/**
	 * See if the provided id is the one that generates the current key
	 * @param id the studentid to be validated
	 */
	public boolean validate (String id) {
		String newKey = generateKey(id);
		
		if (newKey.equals(key)) {
			return true;
		} else {
			return false;
		}
	}
	
	public String getId() {
		return studentId;
	}
}
