/**
 * CBOS - Coffe Bean Ordering System
 * ICS52 - Software Engineering - University of California - Irvine
 * Fall 2000
 * 
 * @author Arvind Krishna
 * @author Roberto Silveira Silva Filho 
 * @version 1.0
 *
 */

package cbos.server;

import cbos.adt.*;
import java.util.Date;
import java.util.Vector;
import cbos.exceptions.*;

/**
 * Stores the current snapshot of the inventory.
 */
public class Inventory implements InventoryInterface, java.io.Serializable {
	
	Vector inventoryList = null; // Sorted list of all orders: sorted by kind, quality, date, #bags
	
	public Inventory() {
		if (inventoryList == null)
			inventoryList = new Vector(0,1);
	}
	
	public void addInventory(String PONumber, int bags, String kind, QualityADT quality, Date date) {
		
		InventoryADT current; 
		InventoryADT newOrder;
		
		// Inserts the elements ordered by kind, quality, date
		
		// Search for same kind
		int size = inventoryList.size();
		int t=0;
		
		if (size >0) {
			
			current = (InventoryADT) inventoryList.elementAt(t);
			
			// Walks until the end of the same kind coffee
			while ( ((current.getKind()).compareTo(kind) < 0) && (t<size) ) {
				current = (InventoryADT) inventoryList.elementAt(t);
				t++;
			}
			
			// If the qualities are the same,
			// walks until the end of the qualities list
			while ( ((current.getQuality()).lessThan(quality)) &&
					((current.getKind()).equals(kind)) &&
					(t<size) ) {
				current = (InventoryADT) inventoryList.elementAt(t);
				t++;
			}
			
			while ( (current.getDate().getTime() < date.getTime()) && (t<size) ) {
				current = (InventoryADT) inventoryList.elementAt(t);
				t++;
			}
		}
		
		newOrder = new InventoryADT(kind, quality, date, bags, 0);
		
		if (t == size) {
			inventoryList.addElement(newOrder); //Append at the end
		} else {
			inventoryList.insertElementAt(newOrder,t);
		}
	}

	/**
	 * Removes the provided ammount of coffee from the inventory
	 * @exception generates insuficient stock exception if there is not enoth stock in the inventory
	 * the exception contains the number of bags in stock.
	 */
	public void removeInventory(String SOnumber, int bags, String kind, QualityADT quality, Date date) throws  InsuficientStockException {
		// If the number of bags = 0 in the inventory list, remove from inventory.
		
		int availableBags = checkAvailability(kind, quality); 
		InventoryADT current;
		
		
		if ( availableBags <= bags ) {
			int t = getFirstReference(kind, quality);
			current = (InventoryADT) inventoryList.elementAt(t);
			int currentBags = current.getBags();
			
			while (bags >0) {
				if (current.getBags() > bags) {
					current.setBags(current.getBags() - bags);
				} else {
					bags = bags - current.getBags();
					inventoryList.removeElementAt(t);
				}
			}
								
		} else {
			throw new InsuficientStockException( (new Integer(availableBags)).toString());
		}
	}
	
	/**
	 * Generates a list of the current inventory, grouped by kind and coffee, showing the oldest coffee first.
	 * @return the ordered list. If the inventory is empty, returns an empty vector.
	 */
	public Vector availabilityList() {
		
		return inventoryList;
	}
	
	/**
	 * Checks for the availability of a kind/quality coffee.
	 * @returns the number of bags, for the provided kind-quality, stored registered in the inventory
	 */
	public int checkAvailability(String kind, QualityADT quality) {
		
		int t;             // Counter
		int size;          // Size of the current inventory list
		int totalBags = 0; // return of the operation
		
		InventoryADT current; 
		
		t = getFirstReference(kind, quality);
		size = inventoryList.size();
		
		// Now add all the coffee bags for the provided quality...
		if (t>=0) {
			current = (InventoryADT) inventoryList.elementAt(t);
			for ( ;t<size; t++) {
				if (current.getKind().equals(kind)) {
					if (current.getQuality().equals(quality)) {
						totalBags += current.getBags();
					}
				
				} else {
					// End of quality in the ordered list...
					break;
				}
			}
		}
		
		return totalBags;
	}
	
	/**
	 * @param kind Name of the kind of the coffee
	 * @quality quality of the coffee searched
	 * @returns the position of the first occurrence of a provided kind/quality in the inventory list.
	 * If the kind was not found, return -1
	 */
	private int getFirstReference(String kind, QualityADT quality) {
		
		int size = inventoryList.size();
		int t=0;
		InventoryADT current = null;
		
		// If not empty vector...
		if (size >0) {
			current = (InventoryADT) inventoryList.elementAt(t);
			
			// Walks until the end of the same kind coffee
			while ( ((current.getKind()).compareTo(kind) != 0) && (t<size) ) {
				current = (InventoryADT) inventoryList.elementAt(t);
				t++;
			}
			
			// Walks until the end of the same kind coffee
			while ( ! current.getQuality().equals(quality) &&
					  ((current.getKind()).equals(kind))&& 
					  (t<size) ) {
				
				current = (InventoryADT) inventoryList.elementAt(t);
				t++;
			}
		}
		
		// If not found...
		if ( (current == null) || (t==size) ||
			 ! current.getQuality().equals(quality) ||
			 ! (current.getKind()).equals(kind)) {
			
			t=-1;
		}
		
		return t;
	}
	
}
