/**
 * CBOS - Coffe Bean Ordering System
 * ICS52 - Software Engineering - University of California - Irvine
 * Fall 2000
 * 
 * @author Arvind Krishna
 * @author Roberto Silveira Silva Filho 
 * @version 1.0
 *
 */
package cbos.server;

import java.io.*;       // Manages file save process
import java.util.zip.*; // Read/write of compressed files

public class Log {
		
	private BugTrack bugTracker;
	
	private String filename = "bugtracker.cbos";
	private String dirname = "./Database/";
	
	private boolean print = false;
	
	/**
	 * Construnctor
	 * @param id the student id
	 * @param numberBugs the number of bugs to track
	 */
	public Log(String id, int numBugs) {
		if (print)
			System.out.println("Log: creating log...");
		
		bugTracker = createBugTracker(id, numBugs);
		saveBugTracker(dirname, filename);
	}
	
	/**
	 * @return true if the submission was ok, false if not.
	 * @param id the student it. It will be validated
	 * @param bugNumber the number of the bug.
	 */
	public boolean submitBug(String id, int bugNumber) {
		
		if (print)
			System.out.println("Log: submitting a bug...");
		
		boolean submissionStatus;
		submissionStatus = bugTracker.submitBug(id, bugNumber);
		
		if (submissionStatus) {
			saveBugTracker(dirname, filename);
		}
		
		return submissionStatus;
	}
	
	/**
	 * Lists the boolean bug array indicating the bugs submitted/not yet submitted 
	 * by the user.
	 */
	public boolean[] listBugsArray () {
		
		if (print)
			System.out.println("Log: listing bugsArray...");
		
		return bugTracker.listBugsArray();
	}
	
	/**
	 * Save the BugTracker module
	 * @param filename the name of the file to be saved
	 * @param dirname the directory to hold the serialized file
	 */
	private void saveBugTracker (String dir, String filename) {
		
		saveObject (bugTracker, filename, dir);
	}
	
	
	/**
	 * Save a provided object to a file.
	 * @param object the object reference to be serialized
	 * @param filename the name of the file to receive the serialized stream data
	 * @param dirname the name of the directory to hold the file
	 */
	private void saveObject(Object object, String filename, String dirname) {
			
		if (print)
			System.out.println("Log: savind object...");
		
		if (filename != null) {
			
			try {

				File directory = new File(dirname);
				if (! directory.exists()) {
					directory.mkdir();
				}
				
				File file = new File(directory, filename);
				FileOutputStream fos = new FileOutputStream(file);
				GZIPOutputStream gzos = new GZIPOutputStream(fos);
				ObjectOutputStream out = new ObjectOutputStream(gzos);
				
				out.writeObject(object);
				
				out.flush();
				out.close();
				
			} catch ( IOException e ) {
				System.out.println("Log: Error while serializing to file: "+dirname+filename);
				System.out.println(e);
			}
		}
	}
	
	
	/**
	 * Loads the BugTracker module from a file/directory or creates a new one
	 * if the file do not exist or is not provided
	 * @param filename the name of the file to load the module
	 * @param dirname the name of the directory holding the module
	 */
	private BugTrack createBugTracker(String id, int numBugs) {
		
		if (print)
			System.out.println("Log: creating bugTracker...");
		
		BugTrack BugTrackerObject = null;
		File file = null;
		
		if (filename != null)
			file = new File ( new File(dirname), filename);
	
		if ( (file != null) && file.exists() ) {			
			
			if (print) 
				System.out.println("Log: Opening file "+file);
			
			try {
				
				FileInputStream fos = new FileInputStream(file);
				GZIPInputStream gzos = new GZIPInputStream(fos);
				ObjectInputStream in = new ObjectInputStream(gzos);
				
				try {				

					if (print) 
						System.out.println("Log: Reading object...");
					
					BugTrackerObject = (BugTrack) in.readObject();
					in.close();
				} catch (ClassNotFoundException ex) {
					System.out.println("Log: Error while reading class in serialized file: "+dirname+filename);
					System.out.println(ex);
				}
				
			} catch ( IOException ex ) {
				System.out.println("Log: Error while reading file: "+dirname+filename);
				System.out.println(ex);
			}
			
		} else {
			
			if (print) 
				System.out.println("Log: Creating new BugTracker object... ");
			
			BugTrackerObject = new BugTrack(id, numBugs);
		}
		
		return BugTrackerObject;
	}
	
	/**
	 * Validates a use login according to the uid stored in the file.
	 */
	public boolean validate (String id) {
		return bugTracker.validate(id);
	}
	
	/**
	 * Gets the user id stored in the file.
	 */
	public String getId () {
		return bugTracker.getId();
	}
}

	