/**
 * CBOS - Coffe Bean Ordering System
 * ICS52 - Software Engineering - University of California - Irvine
 * Fall 2000
 * 
 * @author Arvind Krishna
 * @author Roberto Silveira Silva Filho 
 * @version 1.0
 *
 */

package cbos.server;

import cbos.adt.*;
import java.util.*;
import javax.swing.*;
import java.io.Serializable;

/**
 * Stores and manages information about normal purchase orders.
 * Allows the addition, removal and query of these orders
 */
public class NormalPurchaseOrder implements NormalPurchaseOrderInterface, java.io.Serializable
{
	private Hashtable purchaseOrderTable = null; // Table of purchase orders adts
	private int purchaseOrderCounter;
	
	private transient GrowerInterface grower = null;   // Reference to grower module
	
	private boolean print = false;
	
	/**
	 * Constructor
	 */
	public NormalPurchaseOrder (GrowerInterface gi) {
		
		if (purchaseOrderTable == null) {
			purchaseOrderTable = new Hashtable();
			purchaseOrderCounter = 1;
		}
		
		if (print) {
			System.out.println("NormalPurchaseOrder: Calling constructor...");
			System.out.println("NormalPurchaseOrder: Hashtable.isEmpty() :"+purchaseOrderTable.isEmpty());
		}
		
		grower = gi;
		
		
	}
	
	/**
	 * Called when the file is read from the serialized database
	 */
	public void reconfigure(GrowerInterface gi) {
		grower = gi;
		
	}
	
	/**
	 * Returns the current purchase order counter, used to generate purchase
	 * order numbers
	 * @return current purchase order number
	 */
	public int getCurrentPurchaseOrderCounter() {
		return purchaseOrderCounter;
	}
	
	/**
	 * Adds a new order to the table
	 * @return the purchase order number inserted
	 */
	public String addNormalPurchaseOrder(PurchaseOrderADT purchase) {
		
		if (print)
			System.out.println("NormalPurchaseOrder: adding normal purchase order...");
		
		String purchaseOrderNumber = "PO"+purchaseOrderCounter;
		purchase.setOrderNumber(purchaseOrderNumber);
		purchaseOrderCounter ++;
		purchaseOrderTable.put(purchaseOrderNumber,purchase);
		
		return purchaseOrderNumber;
	}
	
	/**
	 * Updates an order with the actual shipments from the warehouse
	 * @param shipment list form the sales warehouse and the PO number to update the order
	 */
	public boolean updatePurchaseOrder(ShipmentReceiveADT shipment, String  PONumber)
	{
		
		Vector shipment_list = null;
		boolean success = true;
		if(print)
		{
			System.out.println("PurchaseOrder: entering Update order");		
			System.out.println("PurchaseOrder: Normal Po order: "+ PONumber);	
		}
		if (print)
			System.out.println("PurchaseOrder: the size of the Hash table is"+ purchaseOrderTable.size());
		
		PurchaseOrderADT temp = (PurchaseOrderADT) purchaseOrderTable.get(PONumber);
		
		if( temp == null)
		{
			
			String s="There does not exists a Purchase Order with the PurchaseOrderNumber"+PONumber;
			JOptionPane.showMessageDialog(null, s, "ErrorMessage:", JOptionPane.ERROR_MESSAGE);
			success = false;
		}
		else {
			if(print)
			System.out.println("Normal Po, update: PO Exists");
			temp.getShipmentReceiveList().addElement(shipment);
			
			if(print)
				System.out.println("after the comparisn statement");
			
			purchaseOrderTable.put(PONumber, (PurchaseOrderADT)temp);
			success = true;
			
			//inventory.addInventory(PONumber, shipment.getBags(), shipment.getKind(), shipment.getQuality(), shipment.getDate());
			// the above function call adds the new shipment that is delivered in the warehouse into the inventory!
			
		}
		return success;
	}
	
	/**
	 * this operation returns the price of the coffe to the inventory submodule
	 * @param PONumber Purchase order number
	 * @param kind kind of the order 
	 * @param quality quality of the coffee
	 * 
	 */
	public double getPriceDetails(String PONumber, String kind, QualityADT quality)
	{
		String kind_temp;
		int counter =0;
		PurchaseOrderADT temp = (PurchaseOrderADT)purchaseOrderTable.get(PONumber);
		if( temp == null)
			JOptionPane.showMessageDialog(null,"There does not exists an entry in the table", "ErrorMessage", JOptionPane.ERROR_MESSAGE);
		else {
			Vector list = temp.getShipmentReceiveList();
			while( counter <= list.size())
			{
				ShipmentOrderADT shipment = (ShipmentOrderADT) list.elementAt(counter);
				if(( shipment.getKind().equals(kind)) && (shipment.getQuality().equals(quality)))
					return shipment.getPrice();
			}
		}
		return 0;
	}
	
	
	/**
	 * @returns a vector with the scheduled pick-ups for a provided date
	 * @param date date for which the scheduled pick-ups report will be generated.
	 */
	public Vector getScheduledPickUp(Date date)
	{ 
		Vector list = new Vector(); // this contains the details of the scheduled shipment ordered
		Vector shipment_list = null;
		int counter = 0;
		PurchaseOrderADT order = null;
		Enumeration elements ;
		ShipmentOrderADT shipment = null;
		
		if ( purchaseOrderTable.size() != 0)
		{
			elements = purchaseOrderTable.elements();
			while(elements.hasMoreElements())
			{
				order = (PurchaseOrderADT) elements.nextElement();
				if(print)
				{
					System.out.println("the date of the order is " + order.getDate());
					System.out.println("the date is " + date);						
				}
				if(compareDates(date, order.getDate()))
				{
					if(print)
					System.out.println("Inside scheduled:the dates are the same");
					shipment_list = order.getShipmentOrderList();
					counter =0;
					while(counter < shipment_list.size())
					{
						if (print)
							System.out.println("executing scheduled loop for" + counter);
						shipment =  new ShipmentOrderADT();
						ShipmentOrderADT temp = (ShipmentOrderADT) shipment_list.elementAt(counter);
						shipment.setBags(temp.getBags());
						shipment.setKind(temp.getKind());
						shipment.setPrice(temp.getPrice());
						shipment.setQuality(temp.getQuality());
						list.addElement(shipment);
						if (print)
							System.out.println("before adding the value inside scheduled");
						counter++;
					}//end of while
				}//end of if	
			}//end of outer if
			
		} 
		if (print)
			System.out.println("return statement");
		return list;
	}
	
	/**
	 * Lists the actual pickup orders for a given day
	 * @param date to query for actual pick-ups
	 * @return a vector of ShipmentReceiveADTs
	 */
	public Vector getActualPickUp(Date date)
	{ 
		Vector list; // this contains the details of the scheduled shipment ordered
		Vector return_list = new Vector();
		int counter = 0;
		PurchaseOrderADT order;
		Enumeration elements ;
		ShipmentReceiveADT shipment;
		if(purchaseOrderTable.size() != 0)
		{
			elements = purchaseOrderTable.elements();
			
			while(elements.hasMoreElements())
			{
				if (print)
					System.out.println("entered Actual Pick up module");
				order = (PurchaseOrderADT) elements.nextElement();
				list = order.getShipmentReceiveList(); // returns the reference to the list of shipment received list
				if(list != null)
				{
					counter = 0;
					while( counter < list.size())
					{
						shipment = (ShipmentReceiveADT) list.elementAt(counter);
						if(print)
						{
							System.out.println("the date of the order is " + shipment.getDate());
							System.out.println("the date is " + date);						
						}
						if( compareDates(date,shipment.getDate()))
						{
							if (print)
								System.out.println("the dates are the same");
							return_list.addElement(shipment);
						}
						counter++;
					}//end of while
				}//end of if
			}//end of outer while
			
		}	//end of main if 
		
		return return_list; // returns the refernce to the calling program
	}
	
	/**
	 * Gets a list of all purchase orders related to a given grower
	 * @param growerName the name of the grower to be queried
	 * @return a list of ShipmentReceiveADTs
	 */
	public Vector getGrowerPurchases(String growerName)	{
		
		Date date = null;
		PurchaseOrderADT order;
		ShipmentReceiveADT shipment = null;
		int counter = 0, month, prev_month, year, prev_year;
		Vector list;
		Vector return_list = new Vector(0,1);
		Enumeration elements ;
		Calendar currentDate = Calendar.getInstance();
		
		year = currentDate.get(Calendar.YEAR);
		month = currentDate.get(Calendar.MONTH);
		if(purchaseOrderTable.size() > 0)
		{
			elements = purchaseOrderTable.elements();
			while(elements != null && elements.hasMoreElements())
			{
				order = (PurchaseOrderADT) elements.nextElement();
				String tempGrowerName = order.getGrowerName();
				if (tempGrowerName != null && tempGrowerName.equals(growerName)) // check if its the corresponding grower
				{  
					list = order.getShipmentReceiveList();
					if (print)
						System.out.println("inside the purchase per grower\n");
					if(list != null)
					{
						if(print)
						System.out.println("Normal PO:Monthly Purchases: inside list");
						counter = 0;
						while(counter < list.size())
						{ 
							shipment = (ShipmentReceiveADT) list.elementAt(counter);
							date = shipment.getDate();
							prev_year = Integer.parseInt(getYear(date)); // prev year date
							prev_month = getMonth(date); // this is an internal funaction used by fn to get the month
							if(print)
							{
								System.out.println("Normal PO:Monthly Purchases: Prev_month=" + prev_month);
								System.out.println("Normal PO:Monthly Purchases: Prev_year=" + prev_year);						
							}
							if( (month ==1 & prev_month == 12) || (month == prev_month))
							{
								if(print)
									System.out.println("Normal PO:Monthly Purchases: loop1");
								prev_year = Integer.valueOf(getYear(date)).intValue();
								if(prev_year == year -1)   
									return_list.addElement(shipment);
								
							} //end of if
							else	
							{
								if(print)
									System.out.println("Normal PO:Monthly Purchases: loop2");
								if((year == prev_year) && ((prev_month == month -1) || (month == prev_month)))
									return_list.addElement(shipment);
								if (print)
									System.out.println("Order Added to the list");
							}//end of else  */
							counter++;
						} // end of while
					} //end of if
				}// end of the outer if
				
			} // end of the outer while
		} // end of the outmost if statement */
		return return_list;	
	} // end of the function
	
	
	/**
	 * Retursn the month of the year associated with a given date
	 * @return the number of the month
	 */	  
	public int getMonth(Date date)
	{
		String months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
		int i, value =0;
		String temp;
		
		StringTokenizer tokens = new StringTokenizer(date.toString());
		temp = tokens.nextToken();
		temp = tokens.nextToken(); // has the month in the date
		for(i=0; i<12; i++)
			if(months[i].equals(temp))
				value = i+1;
		return(value);
	}
	
	
	public String getYear(Date date)
	{
		String temp = null;
		StringTokenizer tokens = new StringTokenizer(date.toString());
		while( tokens.hasMoreTokens())
		{
			temp = tokens.nextToken();
		}
		return temp;
	}
	
	/**
	 * RPnumber is the repeat purchase order number
	 * @param order order to be inserted (programmed) in the scheduler
	 * @param PRNumber number of repetitions of the order
	 */
	public void scheduleRepeatPurchaseOrder(PurchaseOrderADT order)
	{
		purchaseOrderTable.put(order.getOrderNumber(), (PurchaseOrderADT)order); // it adds the order on to the hash table
		
	} 
	
	/**
	 * @param date date to query for payments
	 * @return the payments that are due to the grower in the form of a list:Vector
	 */
	public Vector getPayments(Date date)
	{
		Vector return_list = new Vector();
		Vector list;
		int counter =0;
		PurchaseOrderADT order;
		ShipmentOrderADT shipment;
		PaymentDetailsADT payments ; // to store the payments to be made on agiven day
		Enumeration elements;
		if(purchaseOrderTable.size() != 0)
		{
			if(print)
				System.out.println("Inside NPO:daily payments:the table is not empty");
			
			elements = purchaseOrderTable.elements();
			while(elements.hasMoreElements())
			{
				order = (PurchaseOrderADT) elements.nextElement();
				if(compareDates(order.getDate(), date))
				{
					if(print)
						System.out.println("Inside NPO:daily payments: the dates are the same");
					list = order.getShipmentOrderList();
					payments = new PaymentDetailsADT();
					payments.setGrowerName(order.getGrowerName());
					if(print)
					{
						System.out.println("the grower refernce is" + grower);
						System.out.println("the grower name is :" + order.getGrowerName());
					}
					if(order.getGrowerName() != null)
						payments.setPaymentAddress(grower.getPaymentAddress(order.getGrowerName()));
					else
						payments.setPaymentAddress(null);
					payments.setOrderList(list);	
					return_list.addElement(payments);
				}
			}
		}// end of the outer if 
		if(print)
			System.out.println("Inside NPO:daily payments: returning the payments");
		return return_list;
	}
	
	public Enumeration getOrderNumbers(){
		Enumeration elements;
		elements = purchaseOrderTable.keys();
		return elements;
	}
	
	/**
	 * compares the two dates for equality : only the day month and the year for similarity
	 * @param the two dates for comparison
	 */
	
	public boolean compareDates(Date date1, Date date2)
	{
		Calendar time1 = Calendar.getInstance();
		Calendar time2 = Calendar.getInstance();
		
		time1.setTime(date1);
		time2.setTime(date2);
		
		if( ( time1.get(Calendar.YEAR) == time2.get(Calendar.YEAR)) && ( time1.get(Calendar.MONTH) == time2.get(Calendar.MONTH)) && ( time1.get(Calendar.DAY_OF_MONTH) == time2.get(Calendar.DAY_OF_MONTH)) )
			return true;
		else
			return false;
		
	} //end of the function
	
	public void setGrower(GrowerInterface gi)
	{
		grower = gi;
		
	}

} // end of the class defenition