/**
 * CBOS - Coffe Bean Ordering System
 * ICS52 - Software Engineering - University of California - Irvine
 * Fall 2000
 * 
 * @author Arvind Krishna
 * @author Roberto Silveira Silva Filho 
 * @version 1.0
 *
 */

package cbos.server;

import cbos.adt.*;
import java.util.*;
import javax.swing.*;
import java.io.Serializable;

/**
 * Stores and manages information about normal sales orders.
 * Allows the addition, removal and query of these orders
 */
public class NormalSalesOrder implements NormalSalesOrderInterface, java.io.Serializable
{
	private Hashtable salesOrderTable = null; // Table of sales orders adts
	private int salesOrderCounter;
	
	private transient CustomerInterface customer = null;   // Reference to customer module
	
	private boolean print = false;
	
	/**
	 * Constructor
	 */
	public NormalSalesOrder (CustomerInterface ci) {
		
		if (salesOrderTable == null) {
			salesOrderTable = new Hashtable();
			salesOrderCounter = 1;
		}
		
		if (print) {
			System.out.println("NormalSalesOrder: Calling constructor...");
			System.out.println("NormalSalesOrder: Hashtable.isEmpty() :"+salesOrderTable.isEmpty());
		}
		customer = ci;
		
	
	}
	
	/**
	 * Returns the order counter 
	 */
	public int getCurrentSalesOrderCounter() {
		return salesOrderCounter;
	}
	
	/**
	 * Reconfigures the server after being read from serialization
	 */
	public void reconfigure (CustomerInterface ci) {
		customer = ci;
		
	}
	
	/**
	 * Adds a new order to the table
	 * @return the sales order number inserted
	 */
	public String addNormalSalesOrder(SalesOrderADT sales) {
		
		if (print)
			System.out.println("NormalSalesOrder: adding normal sales order...");
		
		String salesOrderNumber = "SO"+salesOrderCounter;
		sales.setOrderNumber(salesOrderNumber);
		salesOrderCounter ++;
		salesOrderTable.put(salesOrderNumber, sales);
		
		return salesOrderNumber;
	}
	
	/**
	 * Updates an order with the actual shipments from the warehouse
	 * @param shipment list form the sales warehouse and the PO number to update the order
	  */
	public boolean updateSalesOrder(ShipmentReceiveADT shipment, String  PONumber)
	{
		
		Vector shipment_list = null;
		boolean success = true;
		if(print)
		{
			System.out.println("NormalSalesOrder: Entering update order...");		
			System.out.println("NormalSalesOrder: Normal PO order: "+ PONumber);	
		}
		if (print)
			System.out.println("NormalSalesOrder: the size of the Hash table is"+ salesOrderTable.size());
		SalesOrderADT temp = (SalesOrderADT) salesOrderTable.get(PONumber);
		if( temp == null)
		{
			
			String s="There does not exists a Sales Order with the SalesOrderNumber"+PONumber;
			JOptionPane.showMessageDialog(null, s, "ErrorMessage:", JOptionPane.ERROR_MESSAGE);
			success = false;
		}
		else {
				if(print)
			System.out.println("NormalSalesOrder: Normal Po, update: PO Exists");
			temp.getShipmentReceiveList().addElement(shipment);
			
			if(print)
				System.out.println("NormalSalesOrder:after the comparisn statement");
			
			salesOrderTable.put(PONumber, temp);
			success = true;
			
			//inventory.addInventory(PONumber, shipment.getBags(), shipment.getKind(), shipment.getQuality(), shipment.getDate());
			// the above function call adds the new shipment that is delivered in the warehouse into the inventory!
			
		}
		return success;
	}
			
	/**
	 * this operation returns the price of the coffe to the inventory submodule
	 * @param PONumber Sales order number
	 * @param kind kind of the order 
	 * @param quality quality of the coffee
	 * 
	 */
	public double getPriceDetails(String PONumber, String kind, QualityADT quality)
	{
		String kind_temp;
		int counter =0;
		SalesOrderADT temp = (SalesOrderADT) salesOrderTable.get(PONumber);
		if( temp == null)
			JOptionPane.showMessageDialog(null,"There does not exists an entry in the table", "ErrorMessage", JOptionPane.ERROR_MESSAGE);
		else {
			Vector list = temp.getShipmentReceiveList();
			while( counter <= list.size())
			{
				ShipmentOrderADT shipment = (ShipmentOrderADT) list.elementAt(counter);
				if(( shipment.getKind().equals(kind)) && (shipment.getQuality().equals(quality)))
					return shipment.getPrice();
			}
		}
		return 0;
	}
				
	
	/**
	 * @returns a vector with the scheduled pick-ups for a provided date
	 * @param date date for which the scheduled pick-ups report will be generated.
	 */
	public Vector getScheduledPickUp(Date date)
	{ 
		Vector list = new Vector(); // this contains the details of the scheduled shipment ordered
		Vector shipment_list = null;
		int counter = 0;
		SalesOrderADT order = null;
		Enumeration elements ;
		ShipmentOrderADT shipment = null;
		
		if ( salesOrderTable.size() != 0)
		{
			elements = salesOrderTable.elements();
			while(elements.hasMoreElements())
			{
				order = (SalesOrderADT) elements.nextElement();
					if(print)
						{
							System.out.println("NormalSalesOrder: the date of the order is " + order.getDate());
							System.out.println("NormalSalesOrder: the date is " + date);						
						}
				if(compareDates(date, order.getDate()))
				{
					if(print)
					System.out.println("NormalSalesOrder: Inside scheduled:the dates are the same.");
					shipment_list = order.getShipmentOrderList();
					while(counter < shipment_list.size()) {
						
						if (print)
							System.out.println("NormalSalesOrder: executing scheduled loop for " + counter);
						
						shipment =  new ShipmentOrderADT();
						ShipmentOrderADT temp = (ShipmentOrderADT) shipment_list.elementAt(counter);
						shipment.setBags(temp.getBags());
						shipment.setKind(temp.getKind());
						shipment.setPrice(temp.getPrice());
						shipment.setQuality(temp.getQuality());
						list.addElement(shipment);
						
						if (print)
							System.out.println("NormalSalesOrder: before adding the value inside scheduled.");
						
						counter++;
					
					}//end of while
				}//end of if	
			}//end of outer if
		
		} 
		if (print)
			System.out.println("return statement");
		return list;
	}
	
	/**
	 * Lists the actual pickup orders for a given day
	 * @param date to query for actual pick-ups
	 * @return a vector of ShipmentReceiveADTs
	 */
	public Vector getActualPickUp(Date date)
	{ 
		Vector list; // this contains the details of the scheduled shipment ordered
		Vector return_list = new Vector();
		int counter = 0;
		SalesOrderADT order;
		Enumeration elements ;
		ShipmentReceiveADT shipment;
		if(salesOrderTable.size() != 0)
		{
			elements = salesOrderTable.elements();
		
			while(elements.hasMoreElements())
			{
				if (print)
					System.out.println("entered Actual Pick up module");
				order = (SalesOrderADT) elements.nextElement();
				list = order.getShipmentReceiveList(); // returns the reference to the list of shipment received list
				if(list != null)
				{
					while( counter < list.size())
					{
						shipment = (ShipmentReceiveADT) list.elementAt(counter);
						if(print)
						{
							System.out.println("the date of the order is " + shipment.getDate());
							System.out.println("the date is " + date);						
						}
						if( compareDates(date,shipment.getDate()))
						{
							if (print)
								System.out.println("the dates are the same");
			  				return_list.addElement(shipment);
						}
						counter++;
	  				}//end of while
				}//end of if
			}//end of outer while
		
		}	//end of main if 
	
		return return_list; // returns the refernce to the calling program
	}
	
	/**
	 * Gets a list of all sales orders related to a given customer
	 * @param customerName the name of the customer to be queried
	 * @return a list of ShipmentReceiveADTs
	 */
	public Vector getCustomerSales(String customerName)	{
		
		Date date = null;
		SalesOrderADT order;
		ShipmentReceiveADT shipment = null;
		int counter = 0, month, prev_month, year, prev_year;
		Vector list;
		Vector return_list = new Vector();
		Enumeration elements ;
		Calendar currentDate = Calendar.getInstance();
				
		year = currentDate.get(Calendar.YEAR);
		month = currentDate.get(Calendar.MONTH);
		if(salesOrderTable.size() > 0)
		{
			elements = salesOrderTable.elements();
			while(elements != null && elements.hasMoreElements())
			{
				order = (SalesOrderADT) elements.nextElement();
				String tempCustomerName = order.getCustomerName();
				if (tempCustomerName != null && tempCustomerName.equals(customerName)) // check if its the corresponding customer
				{  
					list = order.getShipmentReceiveList();
					if (print)
						System.out.println("inside the sales per customer\n");
					if(list != null)
					{
					while(counter < list.size())
					{ 
						shipment = (ShipmentReceiveADT) list.elementAt(counter);
						date = shipment.getDate();
						prev_year = Integer.parseInt(getYear(date)); // prev year date
						prev_month = getMonth(date); // this is an internal funaction used by fn to get the month
						
						if( (month ==1 & prev_month == 12) | (month == prev_month))
						{
							prev_year = Integer.valueOf(getYear(date)).intValue();
							if(prev_year == year -1)   
								return_list.addElement(shipment);
							
						} //end of if
						else	
						{
							if((year == prev_year) && ((prev_month == month -1) || (month == prev_month)))
								return_list.addElement(shipment);
							if (print)
								System.out.println("Order Added to the list");
						}//end of else  */
						counter++;
					} // end of while
					} //end of if
				}// end of the outer if
				
			} // end of the outer while
		} // end of the outmost if statement */
			return return_list;	
	} // end of the function
	
	
	/**
	 * Retursn the month of the year associated with a given date
	 * @return the number of the month
	 */	  
	public int getMonth(Date date)
	{
		String months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
		int i, value =0;
		String temp;
		
		StringTokenizer tokens = new StringTokenizer(date.toString());
		temp = tokens.nextToken();
		temp = tokens.nextToken(); // has the month in the date
		for(i=0; i<12; i++)
			if(months[i].equals(temp))
				value = i+1;
		return(value);
	 }
	
	
	public String getYear(Date date)
	{
		String temp = null;
		StringTokenizer tokens = new StringTokenizer(date.toString());
		while( tokens.hasMoreTokens())
		{
			temp = tokens.nextToken();
		}
		return temp;
	}
	
	/**
	 * Places an order generated by the scheduler.
	 * RPnumber is the repeat sales order number
	 * @param order order to be inserted (programmed) in the scheduler
	 * @param PRNumber number of repetitions of the order
	 */
	public void scheduleRepeatSalesOrder(SalesOrderADT order)
	{
		salesOrderTable.put(order.getOrderNumber(), (SalesOrderADT)order); // it adds the order on to the hash table
	
	} 
	
		
	/**
	 * @returns a vector with the scheduled pick-ups for a provided date
	 * @param date date for which the scheduled pick-ups report will be generated.
	 */
	public Vector getScheduledShipments(Date date) {
		
		
		if (print)
			System.out.println("NormalSalesOrder: getScheduledShipments called...");
		
		
		Vector list = new Vector(); // this contains the details of the scheduled shipment ordered
		Vector shipment_list = null;
		int counter = 0;
		SalesOrderADT order = null;
		Enumeration elements ;
		ShipmentOrderADT shipment = null;
		
		if ( salesOrderTable.size() != 0)
		{
			elements = salesOrderTable.elements();
			while(elements.hasMoreElements())
			{
				order = (SalesOrderADT) elements.nextElement();
				if(compareDates(order.getDate(), date))
				{
					shipment_list = order.getShipmentOrderList();
					while(counter < shipment_list.size())
					{
						shipment = (ShipmentOrderADT) shipment_list.elementAt(counter);
						list.addElement(shipment);
					}
				}	
			}
			
		} 
		return list;
	}
	
	
	public ShipmentReceiveADT getCustomerDetails(String customerName) {
		
		
		if (print)
			System.out.println("NormalSalesOrder: getCustomerDetails called...");
		
		return null;
	}
	
	/**
	 * @param date date to query for billings
	 * @return the billings that are due to the customer in the form of a list:Vector
	 */
	public Vector getBillings(Date date)
	{
		Vector return_list = new Vector();
		Vector list;
		int counter =0;
		SalesOrderADT order;
		ShipmentOrderADT shipment;
		BillingDetailsADT billings = new BillingDetailsADT(); // to store the billings to be made on agiven day
		Enumeration elements;
		if(salesOrderTable.size() != 0)
		{
			elements = salesOrderTable.elements();
			while(elements.hasMoreElements())
			{
				order = (SalesOrderADT) elements.nextElement();
				if(compareDates(order.getDate(), date))
				{
					list = order.getShipmentOrderList();
					billings.setCustomerName(order.getCustomerName());
					billings.setBillingAddress(customer.getBillingAddress(order.getCustomerName()));
					billings.setOrderList(list);
					return_list.addElement(billings);
				}
			}
		}// end of the outer if 
		return return_list;
	}
	
	public Enumeration getOrderNumbers(){
		Enumeration elements;
		elements = salesOrderTable.keys();
		return elements;
	}
	
	/**
	 * compares the two dates for equality : only the day month and the year for similarity
	 * @param the two dates for comparison
	 */
	
	public boolean compareDates(Date date1, Date date2)
	{
		Calendar time1 = Calendar.getInstance();
		Calendar time2 = Calendar.getInstance();
		
		time1.setTime(date1);
		time2.setTime(date2);
		
		if( ( time1.get(Calendar.YEAR) == time2.get(Calendar.YEAR)) && ( time1.get(Calendar.MONTH) == time2.get(Calendar.MONTH)) && ( time1.get(Calendar.DAY_OF_MONTH) == time2.get(Calendar.DAY_OF_MONTH)) )
			 return true;
		else
			return false;
		
	} //end of the function
	

} // end of the class defenition