/**
 * CBOS - Coffe Bean Ordering System
 * ICS52 - Software Engineering - University of California - Irvine
 * Fall 2000
 * 
 * @author Arvind Krishna
 * @author Roberto Silveira Silva Filho 
 * @version 1.0
 *
 */

package cbos.server;

import cbos.adt.*;
import java.util.*;

public class RPScheduler implements RPSchedulerInterface, java.io.Serializable 
{
	private Vector list; // vector with scheduled POs for the current data
	private Enumeration elements; // Auxiliary variable
	private int counter; // auxiliary counter
	private transient RepeatPurchaseOrderInterface rpi; // holds the reference of Repeat Purchase Interface
	private transient NormalPurchaseOrderInterface npi; // interface to convert the repeat orders into normal orders
	private Date currentDate;
	private PurchaseOrderADT normalOrder;
	private RepeatPurchaseOrderADT repeatPurchase;
	private boolean print = false;
	private Date date;
	
	/**
	 * Constructor
	 * @param repeatPO Reference to the repeat purchase order database
	 * @param normalPO Reference to the normal purchase order database
	 */
	public RPScheduler( RepeatPurchaseOrderInterface repeatPO, NormalPurchaseOrderInterface normalPO)
	{
		
		
		this.rpi = repeatPO; // assigning the references to 
		this.npi = normalPO;
		if(print)
			System.out.println("RPScheduler Constructor ");
	}
	
	/**
	 * Reconfigure the references to repeatPO and normalPO after 
	 * readin a serialized file
	 */
	public void reConfigure ( RepeatPurchaseOrderInterface repeatPO, NormalPurchaseOrderInterface normalPO) {
		if(print)
			System.out.println("RPScheduler: Reconfiguring...");
		
		this.rpi = repeatPO; // assigning the references to 
		this.npi = normalPO;
		
	}
	
	/**
	 * @param this is called to run every day and check for the condition
	 */
	public void run()
	{
	
		Calendar cln = Calendar.getInstance();
		cln.add(Calendar.DATE, +10); // add tendays fomr now the order is due for shipping
		date = cln.getTime();
		currentDate = new Date();
		normalOrder = new PurchaseOrderADT();
		try
		{
		list = rpi.checkDueRepeatOrder(currentDate);
		}catch(Exception e)
		{	System.out.println(e); }
		if(list != null)
		{
			elements = list.elements();
			while( (elements != null) && elements.hasMoreElements())
			{
				repeatPurchase = (RepeatPurchaseOrderADT) elements.nextElement();
				normalOrder.setGrowerName(repeatPurchase.getGrowerName());
				normalOrder.setPickUpOption(repeatPurchase.getPickUpOption());
				normalOrder.setPickUpAddress(repeatPurchase.getPickUpAddress());
				normalOrder.setOrderNumber(repeatPurchase.getOrderNumber()); // this has the number as rp1234-1
				normalOrder.setShipmentOrderList(repeatPurchase.getShipmentOrderList());
				normalOrder.setShipmentReceiveList(repeatPurchase.getShipmentReceiveList());
				normalOrder.setDate(date); // set the date when the order is expected in the system as 1o days from now
				npi.scheduleRepeatPurchaseOrder(normalOrder); // adds it to the  normal order
			}
		  }
		}//end of the constructor
	
}// end of the class
