/**
 * CBOS - Coffe Bean Ordering System
 * ICS52 - Software Engineering - University of California - Irvine
 * Fall 2000
 * 
 * @author Arvind Krishna
 * @author Roberto Silveira Silva Filho 
 * @version 1.0
 *
 */

package cbos.server;

import java.io.*;       // Manages file save process
import java.util.zip.*; // Read/write of compressed files

public class ServerFactory implements ServerFactoryInterface {
	
	private boolean print = false; // Togles on/of the trace
	
	private GrowerInterface grower								= null;
	private KindInterface kind									= null;
	private UserInterface user									= null;
	private CustomerInterface customer							= null;
	private InventoryInterface inventory						= null;
	private RPSchedulerInterface rpScheduler					= null;
	private RSSchedulerInterface rsScheduler					= null;
	private NormalPurchaseOrderInterface normalPurchaseOrder	= null;
	private RepeatPurchaseOrderInterface repeatPurchaseOrder	= null;
	private NormalSalesOrderInterface normalSalesOrder			= null;
	private RepeatSalesOrderInterface repeatSalesOrder			= null;
	private Log log = null; // Reference to the log manager (bugtracker)
	
	private String login = ""; // the user login
	
	public static final String growerFile	        = "grower.cbos";
	public static final String kindFile		        = "kind.cbos";
	public static final String userFile		        = "user.cbos";
	public static final String customerFile         = "customer.cbos";
	public static final String inventoryFile        = "inventory.cbos";
	public static final String rpSchedulerFile      = "rpScheduler.cbos";
	public static final String rsSchedulerFile      = "rsScheduler.cbos";
	public static final String normalPurchaseFile   = "normalPurchase.cbos";
	public static final String repeatPurchaseFile   = "repeatPurchase.cbos";
	public static final String normalSalesFile	    = "normalSales.cbos";
	public static final String repeatSalesFile	    = "repeatSales.cbos";
	
	public static final String dirname = "./Database/";
	
	
	/**
	 * Creates the server side and the log object for the provides student id, with the provideed number of bugs
	 * @param uid the user id number
	 * @param numberBugs the number of bugs to be tracked.
	 */
	public void createServerSide(String uid, int numberBugs) {
		createServerSide();
		log = new Log(uid,numberBugs);
	}
	
	/**
	 * Creates the server side strucrure
	 */
	public void createServerSide () {
		
		if (print)
			System.out.println("ServerFactory: creating server side...");
		
		grower    = createGrower (growerFile, dirname);
		kind      = createKind (kindFile, dirname);
		user      = createUser (userFile, dirname);
		customer  = createCustomer (customerFile, dirname);
		inventory = createInventory (inventoryFile, dirname);
		repeatPurchaseOrder = createRepeatPurchaseOrder(repeatPurchaseFile, dirname);
		normalPurchaseOrder = createNormalPurchaseOrder(normalPurchaseFile, dirname, grower);
		repeatSalesOrder	= createRepeatSalesOrder(repeatSalesFile, dirname);
		normalSalesOrder	= createNormalSalesOrder(normalSalesFile, dirname, customer);
		rpScheduler = createRPScheduler (rpSchedulerFile, dirname, repeatPurchaseOrder, normalPurchaseOrder);
		rsScheduler = createRSScheduler (rsSchedulerFile, dirname, repeatSalesOrder, normalSalesOrder);

		if (print)
			System.out.println("ServerFactory: server side CREATED");

	}	
	
	public void saveAll() {
		
		if (print)
			System.out.println("ServerFactory: saving all files...");
		
		saveGrower(dirname, growerFile);
		saveKind(dirname, kindFile);
		saveUser(dirname, userFile);
		saveCustomer(dirname, customerFile);
		saveInventory(dirname, inventoryFile);
		saveRepeatPurchaseOrder(dirname, repeatPurchaseFile);
		saveNormalPurchaseOrder(dirname, normalPurchaseFile);
		saveRepeatSalesOrder(dirname, repeatSalesFile);
		saveNormalSalesOrder(dirname, normalSalesFile);
		saveRPScheduler(dirname, rpSchedulerFile);
		saveRSScheduler(dirname, rsSchedulerFile);
		
		if (print)
			System.out.println("ServerFactory: DONE saving all files...");
		
	
	}
	
	/**
	 * Save the grower module
	 * @param filename the name of the file to be saved
	 * @param dirname the directory to hold the serialized file
	 */
	public void saveGrower (String dir, String filename) {
		
		if (dir == null)
			dir = dirname;
		
		if (filename == null)
			filename = growerFile;
		
		saveObject (grower, filename, dir);
	}
	
	/**
	 * Save the kind module
	 * @param filename the name of the file to be saved
	 * @param dirname the directory to hold the serialized file
	 */
	public void saveKind (String dir, String filename) {
		
		if (dir == null)
			dir = dirname;
		
		if (filename == null)
			filename = kindFile;
		
		
		saveObject (kind, filename, dir);
	}
	
	/**
	 * Save the user module
	 * @param filename the name of the file to be saved
	 * @param dirname the directory to hold the serialized file
	 */
	public void saveUser (String dir, String filename) {
		
		if (dir == null)
			dir = dirname;
		
		if (filename == null)
			filename = userFile;
		
		saveObject (user, filename, dir);
	}
	
	/**
	 * Save the Customer module
	 * @param filename the name of the file to be saved
	 * @param dirname the directory to hold the serialized file
	 */
	public void saveCustomer (String dir, String filename) {
		
		if (dir == null)
			dir = dirname;
		
		if (filename == null)
			filename = customerFile;
		
		saveObject (customer, filename, dir);
	}
	
	/**
	 * Save the inventory module
	 * @param filename the name of the file to be saved
	 * @param dirname the directory to hold the serialized file
	 */
	public void saveInventory (String dir, String filename) {
		
		if (dir == null)
			dir = dirname;
		
		if (filename == null)
			filename = inventoryFile;
		
		saveObject (inventory, filename, dir);
	}
	
	/**
	 * Save the RSScheduler module
	 * @param filename the name of the file to be saved
	 * @param dirname the directory to hold the serialized file
	 */
	public void saveRSScheduler (String dir, String filename) {
		
		if (dir == null)
			dir = dirname;
		
		if (filename == null)
			filename = rsSchedulerFile;
		
		saveObject (rsScheduler, filename, dir);
	}
	
	/**
	 * Save the RPScheduler module
	 * @param filename the name of the file to be saved
	 * @param dirname the directory to hold the serialized file
	 */
	public void saveRPScheduler (String dir, String filename) {
		if (dir == null)
			dir = dirname;
		
		if (filename == null)
			filename = rpSchedulerFile;
		
		saveObject (rpScheduler, filename, dir);
	}
	
	/**
	 * Save the RepeatPurchaseOrder module
	 * @param filename the name of the file to be saved
	 * @param dirname the directory to hold the serialized file
	 */
	public void saveRepeatPurchaseOrder (String dir, String filename) {
		if (dir == null)
			dir = dirname;
		
		if (filename == null)
			filename = repeatPurchaseFile;
		
		saveObject (repeatPurchaseOrder, filename, dir);
	}
	
	/**
	 * Save the NormalPurchaseOrder module
	 * @param filename the name of the file to be saved
	 * @param dirname the directory to hold the serialized file
	 */
	public void saveNormalPurchaseOrder (String dir, String filename) {
		if (dir == null)
			dir = dirname;
		
		if (filename == null)
			filename = normalPurchaseFile;
		
		saveObject (normalPurchaseOrder, filename, dir);
	}

	/**
	 * Save the RepeatSalesOrder module
	 * @param filename the name of the file to be saved
	 * @param dirname the directory to hold the serialized file
	 */
	public void saveRepeatSalesOrder (String dir, String filename) {
		
		if (dir == null)
			dir = dirname;
		
		if (filename == null)
			filename = repeatSalesFile;
		
		saveObject (repeatSalesOrder, filename, dir);
	}
	
	/**
	 * Save the NormalSalesOrder module
	 * @param filename the name of the file to be saved
	 * @param dirname the directory to hold the serialized file
	 */
	public void saveNormalSalesOrder (String dir, String filename) {
		if (dir == null)
			dir = dirname;
		
		if (filename == null)
			filename = normalSalesFile;
		
		saveObject (normalSalesOrder, filename, dir);
	}



	
	
	/**
	 * Save a provided object to a file.
	 * @param object the object reference to be serialized
	 * @param filename the name of the file to receive the serialized stream data
	 * @param dirname the name of the directory to hold the file
	 */
	private void saveObject(Object object, String filename, String dirname) {
			
		if (filename != null) {
			
			try {

				File directory = new File(dirname);
				if (! directory.exists()) {
					directory.mkdir();
				}
				
				File file = new File(directory, filename);
				FileOutputStream fos = new FileOutputStream(file);
				GZIPOutputStream gzos = new GZIPOutputStream(fos);
				ObjectOutputStream out = new ObjectOutputStream(gzos);
				
				out.writeObject(object);
				
				out.flush();
				out.close();
				
			} catch ( IOException e ) {
				System.out.println("LOA: Error while serializing to file: "+dirname+filename);
				System.out.println(e);
			}
		}
	}
	
	/**
	 * Loads the grower module from a file/directory or creates a new one
	 * if the file do not exist or is not provided
	 * @param filename the name of the file to load the module
	 * @param dirname the name of the directory holding the module
	 */
	public GrowerInterface createGrower(String filename, String dirname) {
		
		Grower growerObject = null;
		File file = null;
		
		if (filename != null)
			file = new File ( new File(dirname), filename);
	
		if ( (file != null) && file.exists() ) {			
			
			if (print) 
				System.out.println("ServerFactory: Opening file "+file);
			
			try {
				
				FileInputStream fos = new FileInputStream(file);
				GZIPInputStream gzos = new GZIPInputStream(fos);
				ObjectInputStream in = new ObjectInputStream(gzos);
				
				try {				

					if (print) 
						System.out.println("ServerFactory: Reading object...");
					
					growerObject = (Grower) in.readObject();
					in.close();
				} catch (ClassNotFoundException ex) {
					System.out.println("LOA: Error while reading class in serialized file: "+dirname+filename);
					System.out.println(ex);
				}
				
			} catch ( IOException ex ) {
				System.out.println("CBOS: Error while reading file: "+dirname+filename);
				System.out.println(ex);
			}
			
		} else {
			
			if (print) 
				System.out.println("ServerFactory: Creating new grower object... ");
			
			growerObject = new Grower();
		}
		
		return growerObject;
	}
	
	
	/**
	 * Loads the kind module from a file/directory or creates a new one
	 * if the file do not exist or is not provided
	 * @param filename the name of the file to load the module
	 * @param dirname the name of the directory holding the module
	 */
	public KindInterface createKind(String filename, String dirname) {
	
		if (print)
			System.out.println("ServerFactory: creating kind ...");
		
		Kind kindObject = null;
		
		File file = null;
		
		if (filename != null)
			file = new File ( new File(dirname), filename);
	
		if ( (file != null) && file.exists() ) {			
			
			try {
				
				if (print) 
				System.out.println("ServerFactory: Opening file "+file);
				
				FileInputStream fos = new FileInputStream(file);
				GZIPInputStream gzos = new GZIPInputStream(fos);
				ObjectInputStream in = new ObjectInputStream(gzos);
				
				try {
					
					if (print) 
						System.out.println("ServerFactory: Reading object...");
			
					
					kindObject = (Kind) in.readObject();
					in.close();
					
				} catch (ClassNotFoundException ex) {
					System.out.println("LOA: Error while reading class in serialized file: "+dirname+filename);
					System.out.println(ex);
				}
				
			} catch ( IOException ex ) {
				System.out.println("CBOS: Error while reading file: "+dirname+filename);
				System.out.println(ex);
			}
			
		} else {
			if (print) 
				System.out.println("ServerFactory: Creating new kind object... ");
			
			kindObject = new Kind();
		}
		
		return kindObject;
	}
	
	
	/**
	 * Loads the user module from a file/directory or creates a new one
	 * if the file do not exist or is not provided
	 * @param filename the name of the file to load the module
	 * @param dirname the name of the directory holding the module
	 */
	public UserInterface createUser(String filename, String dirname) {
	
		if (print)
			System.out.println("ServerFactory: creating user ...");
		
		User userObject = null;
		File file = null;
		
		if (filename != null)
			file = new File ( new File(dirname), filename);
	
		if ( (file != null) && file.exists() ) {			
			
			try {
				
				if (print) 
				System.out.println("ServerFactory: Opening file "+file);
				
				FileInputStream fos = new FileInputStream(file);
				GZIPInputStream gzos = new GZIPInputStream(fos);
				ObjectInputStream in = new ObjectInputStream(gzos);
				
				try {
					
					if (print) 
						System.out.println("ServerFactory: Reading object...");
			
					
					userObject = (User) in.readObject();
					in.close();
					
				} catch (ClassNotFoundException ex) {
					System.out.println("LOA: Error while reading class in serialized file: "+dirname+filename);
					System.out.println(ex);
				}
				
			} catch ( IOException ex ) {
				System.out.println("CBOS: Error while reading file: "+dirname+filename);
				System.out.println(ex);
			}
			
		} else {
			if (print) 
				System.out.println("ServerFactory: Creating new user object... ");
			
			userObject = new User();
		}
		
		return userObject;
	}
	
	/**
	 * Loads the Inventory module from a file/directory or creates a new one
	 * if the file do not exist or is not provided
	 * @param filename the name of the file to load the module
	 * @param dirname the name of the directory holding the module
	 */
	public InventoryInterface createInventory(String filename, String dirname) {
		
		if (print)
			System.out.println("ServerFactory: creating inventory...");
		
		
		Inventory InventoryObject = null;
		File file = null;
		
		if (filename != null)
			file = new File ( new File(dirname), filename);
	
		if ( (file != null) && file.exists() ) {			
			
			try {
				
				if (print) 
				System.out.println("ServerFactory: Opening file "+file);
				
				FileInputStream fos = new FileInputStream(file);
				GZIPInputStream gzos = new GZIPInputStream(fos);
				ObjectInputStream in = new ObjectInputStream(gzos);
				
				try {
					
					if (print) 
						System.out.println("ServerFactory: Reading object...");
						
					InventoryObject = (Inventory) in.readObject();
					in.close();
					
				} catch (ClassNotFoundException ex) {
					System.out.println("LOA: Error while reading class in serialized file: "+dirname+filename);
					System.out.println(ex);
				}
				
			} catch ( IOException ex ) {
				System.out.println("CBOS: Error while reading file: "+dirname+filename);
				System.out.println(ex);
			}
			
		} else {
			InventoryObject = new Inventory();
		}
		
		return InventoryObject;
	}
	
	/**
	 * Creates a nes instance of RPScheduler class or reads it form a serialized file
	 * @param filename the name of the file with the class state
	 * @param dirname the name of the directory holding the file
	 * @param repeatPO Reference to the repeated purchase order repository
	 * @param normalPO Reference to the normal purchase order repository
	 */
	public RPSchedulerInterface createRPScheduler(String filename, String dirname, RepeatPurchaseOrderInterface repeatPO, NormalPurchaseOrderInterface normalPO) {
				
		RPScheduler RPSchedulerObject = null;
		File file = null; 
		
		if (print)
			System.out.println("ServerFactory: creating RPScheduler...");
		
		if (filename != null)
			file = new File ( new File(dirname), filename);
	
		if ( (file != null) && file.exists() ) {			
			
			try {
				
				if (print) 
					System.out.println("ServerFactory: Opening file "+file);
				
				FileInputStream fos = new FileInputStream(file);
				GZIPInputStream gzos = new GZIPInputStream(fos);
				ObjectInputStream in = new ObjectInputStream(gzos);
				
				try {
					
					if (print) 
						System.out.println("ServerFactory: Reading object...");
						
					RPSchedulerObject = (RPScheduler) in.readObject();
					in.close();
					
				} catch (ClassNotFoundException ex) {
					System.out.println("LOA: Error while reading class in serialized file: "+dirname+filename);
					System.out.println(ex);
				}
				
			} catch ( IOException ex ) {
				System.out.println("CBOS: Error while reading file: "+dirname+filename);
				System.out.println(ex);
			}
			
			// Give new references to the variables that were not serialized
			RPSchedulerObject.reConfigure(repeatPO, normalPO);
			
		} else {
			
			if (print)
				System.out.println("ServerFactory: creating new object for RPScheduler...");
		
			RPSchedulerObject = new RPScheduler(repeatPO, normalPO);
		}
		
		return RPSchedulerObject;
	}

	
	/**
	 * Creates a nes instance of RSScheduler class or reads it form a serialized file
	 * @param filename the name of the file with the class state
	 * @param dirname the name of the directory holding the file
	 * @param repeatSO Reference to the repeat purchase order database
	 * @param normalSO Reference to the normal purchase order database
	 */
	public RSSchedulerInterface createRSScheduler(String filename, String dirname, RepeatSalesOrderInterface repeatSO, NormalSalesOrderInterface normalSO) {
		
		if (print)
			System.out.println("ServerFactory: creating RSScheduler...");
		
		RSScheduler RSSchedulerObject = null; // Repeated sales scheduler
		File file = null;
		
		if (filename != null)
			file = new File ( new File(dirname), filename);
	
		if ( (file != null) && file.exists() ) {			
			
			try {
				
				if (print) 
				System.out.println("ServerFactory: Opening file "+file);
				
				FileInputStream fos = new FileInputStream(file);
				GZIPInputStream gzos = new GZIPInputStream(fos);
				ObjectInputStream in = new ObjectInputStream(gzos);
				
				try {
					
					if (print) 
						System.out.println("ServerFactory: Reading object...");
						
					RSSchedulerObject = (RSScheduler) in.readObject();
					in.close();
					
				} catch (ClassNotFoundException ex) {
					System.out.println("LOA: Error while reading class in serialized file: "+dirname+filename);
					System.out.println(ex);
				}
				
			} catch ( IOException ex ) {
				System.out.println("CBOS: Error while reading file: "+dirname+filename);
				System.out.println(ex);
			}
			
			RSSchedulerObject.reConfigure(repeatSO, normalSO);
			
		} else {
			RSSchedulerObject = new RSScheduler(repeatSO, normalSO);
		}
		
		return RSSchedulerObject;
	}
	
	/**
	 * Loads the Customer module from a file/directory or creates a new one
	 * if the file do not exist or is not provided
	 * @param filename the name of the file to load the module
	 * @param dirname the name of the directory holding the module
	 */
	public CustomerInterface createCustomer(String filename, String dirname) {
		
		Customer CustomerObject = null;
		File file = null;
		
		if (print)
			System.out.println("ServerFactory: creating Customer...");

		if (filename != null)
			file = new File ( new File(dirname), filename);
	
		if ( (file != null) && file.exists() ) {			
			
			try {
				
				if (print) 
				System.out.println("ServerFactory: Opening file "+file);
				
				FileInputStream fos = new FileInputStream(file);
				GZIPInputStream gzos = new GZIPInputStream(fos);
				ObjectInputStream in = new ObjectInputStream(gzos);
				
				try {
					
					if (print) 
						System.out.println("ServerFactory: Reading object...");
						
					CustomerObject = (Customer) in.readObject();
					in.close();
					
				} catch (ClassNotFoundException ex) {
					System.out.println("LOA: Error while reading class in serialized file: "+dirname+filename);
					System.out.println(ex);
				}
				
			} catch ( IOException ex ) {
				System.out.println("CBOS: Error while reading file: "+dirname+filename);
				System.out.println(ex);
			}
			
		} else {
			CustomerObject = new Customer();
		}
		
		return CustomerObject;
	}

	
	
	/**
	 * Loads the RepeatPurchaseOrder module from a file/directory or creates a new one
	 * if the file do not exist or is not provided
	 * @param filename the name of the file to load the module
	 * @param dirname the name of the directory holding the module
	 */
	public RepeatPurchaseOrderInterface createRepeatPurchaseOrder(String filename, String dirname) {
		
		RepeatPurchaseOrder repeatPurchaseOrderObject = null;
		File file = null;
		
		if (print)
			System.out.println("ServerFactory: creating RepeatPurchaseOrder...");
		
		if (filename != null)
			file = new File ( new File(dirname), filename);
	
		if ( (file != null) && file.exists() ) {			
			
			try {
				
				if (print) 
				System.out.println("ServerFactory: Opening file "+file);
				
				FileInputStream fos = new FileInputStream(file);
				GZIPInputStream gzos = new GZIPInputStream(fos);
				ObjectInputStream in = new ObjectInputStream(gzos);
				
				try {
					
					if (print) 
						System.out.println("ServerFactory: Reading object...");
						
					repeatPurchaseOrderObject = (RepeatPurchaseOrder) in.readObject();
					in.close();
					
				} catch (ClassNotFoundException ex) {
					System.out.println("LOA: Error while reading class in serialized file: "+dirname+filename);
					System.out.println(ex);
				}
				
			} catch ( IOException ex ) {
				System.out.println("CBOS: Error while reading file: "+dirname+filename);
				System.out.println(ex);
			}
			
		} else {
			repeatPurchaseOrderObject = new RepeatPurchaseOrder();
		}
		
		return repeatPurchaseOrderObject;
	}

	
	/**
	 * Loads the NormalPurchaseOrder module from a file/directory or creates a new one
	 * if the file do not exist or is not provided
	 * @param filename the name of the file to load the module
	 * @param dirname the name of the directory holding the module
	 */
	public NormalPurchaseOrderInterface createNormalPurchaseOrder(String filename, String dirname, GrowerInterface growerInt) {
		
		NormalPurchaseOrder normalPurchaseOrderObject = null;
		File file = null;
		
		if (print)
			System.out.println("ServerFactory: creating NormalPurchaseOrder...");
		
		if (filename != null)
			file = new File ( new File(dirname), filename);
	
		if ( (file != null) && file.exists() ) {			
			
			try {
				
				if (print) 
				System.out.println("ServerFactory: Opening file "+file);
				
				FileInputStream fos = new FileInputStream(file);
				GZIPInputStream gzos = new GZIPInputStream(fos);
				ObjectInputStream in = new ObjectInputStream(gzos);
				
				try {
					
					if (print) 
						System.out.println("ServerFactory: Reading object...");
						
					normalPurchaseOrderObject = (NormalPurchaseOrder) in.readObject();
					in.close();
					
				} catch (ClassNotFoundException ex) {
					System.out.println("LOA: Error while reading class in serialized file: "+dirname+filename);
					System.out.println(ex);
				}
				
			} catch ( IOException ex ) {
				System.out.println("CBOS: Error while reading file: "+dirname+filename);
				System.out.println(ex);
			}
			
			normalPurchaseOrderObject.setGrower(growerInt);
			
		} else {
			normalPurchaseOrderObject = new NormalPurchaseOrder(growerInt);
		}
		
		return normalPurchaseOrderObject;
	}

	/**
	 * Loads the RepeatSalesOrder module from a file/directory or creates a new one
	 * if the file do not exist or is not provided
	 * @param filename the name of the file to load the module
	 * @param dirname the name of the directory holding the module
	 */
	public RepeatSalesOrderInterface createRepeatSalesOrder(String filename, String dirname) {
		
		RepeatSalesOrder repeatSalesOrderObject = null;
		File file = null;
		
		if (print)
			System.out.println("ServerFactory: creating RepeatSalesOrder...");
		
		if (filename != null)
			file = new File ( new File(dirname), filename);
	
		if ( (file != null) && file.exists() ) {			
			
			try {
				
				if (print) 
				System.out.println("ServerFactory: Opening file "+file);
				
				FileInputStream fos = new FileInputStream(file);
				GZIPInputStream gzos = new GZIPInputStream(fos);
				ObjectInputStream in = new ObjectInputStream(gzos);
				
				try {
					
					if (print) 
						System.out.println("ServerFactory: Reading object...");
						
					repeatSalesOrderObject = (RepeatSalesOrder) in.readObject();
					in.close();
					
				} catch (ClassNotFoundException ex) {
					System.out.println("LOA: Error while reading class in serialized file: "+dirname+filename);
					System.out.println(ex);
				}
				
			} catch ( IOException ex ) {
				System.out.println("CBOS: Error while reading file: "+dirname+filename);
				System.out.println(ex);
			}
			
		} else {
			repeatSalesOrderObject = new RepeatSalesOrder();
		}
		
		return repeatSalesOrderObject;
	}

	
	/**
	 * Loads the NormalSalesOrder module from a file/directory or creates a new one
	 * if the file do not exist or is not provided
	 * @param filename the name of the file to load the module
	 * @param dirname the name of the directory holding the module
	 */
	public NormalSalesOrderInterface createNormalSalesOrder(String filename, String dirname, CustomerInterface customerInt) {
		
		NormalSalesOrder normalSalesOrderObject = null;
		File file = null;
		
		if (print)
			System.out.println("ServerFactory: creating NormalSalesOrder...");
		
		if (filename != null)
			file = new File ( new File(dirname), filename);
	
		if ( (file != null) && file.exists() ) {			
			
			try {
				
				if (print) 
				System.out.println("ServerFactory: Opening file "+file);
				
				FileInputStream fos = new FileInputStream(file);
				GZIPInputStream gzos = new GZIPInputStream(fos);
				ObjectInputStream in = new ObjectInputStream(gzos);
				
				try {
					
					if (print) 
						System.out.println("ServerFactory: Reading object...");
						
					normalSalesOrderObject = (NormalSalesOrder) in.readObject();
					in.close();
					
				} catch (ClassNotFoundException ex) {
					System.out.println("LOA: Error while reading class in serialized file: "+dirname+filename);
					System.out.println(ex);
				}
				
			} catch ( IOException ex ) {
				System.out.println("CBOS: Error while reading file: "+dirname+filename);
				System.out.println(ex);
			}
			
			normalSalesOrderObject.reconfigure(customerInt);
			
		} else {
			normalSalesOrderObject = new NormalSalesOrder(customerInt);
		}
		
		return normalSalesOrderObject;
	}
	
	
	//***********************************************************************
	
	
	public GrowerInterface getGrower() {
		if (print)
			System.out.println("ServerFactory: returning grower: "+grower);
		
		return grower;
	}
	
	public KindInterface getKind() {
		if (print)
			System.out.println("ServerFactory: returning kind: "+kind);
		
		return kind;
	}
	
	public UserInterface getUser() {
		if (print)
			System.out.println("ServerFactory: returning user: "+user);
		
		return user;
	}
	
	
	public CustomerInterface getCustomer()  {
		if (print)
			System.out.println("ServerFactory: returning customer: "+customer);
		
		return 	customer;
	}
	
	public InventoryInterface getInventory() {
		if (print)
			System.out.println("ServerFactory: returning inventory: "+inventory);
		
		return inventory;
	}
	
	public RPSchedulerInterface getRPScheduler() {
		if (print)
			System.out.println("ServerFactory: returning rpScheduler: "+rpScheduler);
		
		return rpScheduler;
	}
	
	public RSSchedulerInterface getRSScheduler() {
		if (print)
			System.out.println("ServerFactory: returning rsScheduler: "+rsScheduler);
		
		return rsScheduler;
	}
	
	public NormalPurchaseOrderInterface getNormalPurchaseOrder() {
		if (print)
			System.out.println("ServerFactory: returning normalPurchaseOrder: "+normalPurchaseOrder);
		
		return normalPurchaseOrder;
	}
	
	public RepeatPurchaseOrderInterface getRepeatPurchaseOrder() {
		if (print)
			System.out.println("ServerFactory: returning repeatPurchaseOrder: "+repeatPurchaseOrder);
		
		return repeatPurchaseOrder;
	}
	
	public NormalSalesOrderInterface getNormalSalesOrder() {
		if (print)
			System.out.println("ServerFactory: returning normalSalesOrder: "+normalSalesOrder);
		
		return normalSalesOrder;
	}
	
	public RepeatSalesOrderInterface getRepeatSalesOrder() {
		if (print)
			System.out.println("ServerFactory: returning repeatSalesOrder: "+repeatSalesOrder);

		return repeatSalesOrder;
	}
	
	public Log getLog() {
		if (print)
			System.out.println("ServerFactory: returning Log: "+log);

		return log;
	}
	
	public String getLogin() {
		if (print)
			System.out.println("ServerFactory: returning Login: "+login);

		return login;
	}
	
	public void setLogin(String value) {
		if (print)
			System.out.println("ServerFactory: setting Login: "+value);

		login = value;
	}
}
