/**
 * CBOS - Coffe Bean Ordering System
 * ICS52 - Software Engineering - University of California - Irvine
 * Fall 2000
 * 
 * @author Arvind Krishna
 * @author Roberto Silveira Silva Filho 
 * @version 1.0
 *
 */

package cbos.server;

import java.util.*;
import cbos.adt.*;

public class User implements UserInterface, java.io.Serializable 	 
{
	
	private Hashtable userTable = null;
	private boolean print = false;
	
	/**
	 * Constructor
	 */
	public User()
	{
		// Serialization guard
		if (userTable == null)
			userTable = new Hashtable(10, (float) 0.75);
		
		// key in the hashtable = login
	}
	
	public void addUser(String name, String login, UserFunctionADT function) {
	
		if (print) {
			System.out.println("User: adding userDetails...");
			System.out.println("User: name: "+name);
			System.out.println("User: login: "+login);
			System.out.println("User: function: "+function);
		}
		
		UserADT userData = new UserADT();
		userData.setName(name);
		userData.setLogin(login);
		userData.setFunction(function);
		
		// Overrides or inserts a new one
		userTable.put(login, userData);
	
	}
	
	public void addUser(UserADT user) {
	
		if (print) {
			System.out.println("User: adding userDetails...");
			System.out.println("User: name: "+user.getName() );
			System.out.println("User: login: "+user.getLogin() );
			System.out.println("User: function: "+user.getFunction() );
		}
			
		userTable.put(user.getLogin(), user);
	
	}
	
	/**
	 * Remove the user. The key in the hashtable is the login
	 */
	public void removeUser(String login) {
		userTable.remove(login);
	}
	
	/**
	 * Returns the list of all users in the system.
	 * @return a vector with ClientADT elements
	 */
	public Vector getUsersList() {
		
		if (print)
			System.out.println("User: getting user list...");
		
		Vector list = new Vector(0,1);
		
		Enumeration elements;
		elements = userTable.elements();
		
		while( elements.hasMoreElements())
			list.addElement(elements.nextElement());
		
		if (print)
			System.out.println("User: returning: "+list);
		
		return list;
	}
	
	/**
	 * @param login the login of the user being queried
	 * @eturn the information of a given user identified by its login
	 */
	public UserADT getUserDetails(String login) {
		UserADT tempData = (UserADT) userTable.get(login);
		
		return tempData;
	}
	
	/**
	 * Clears the users table.
	 */
	public void clearAll() {
		userTable.clear();
	}
	
} 

